% Compute the irrotational stabilization stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = irrot_stab_stiffness(feb, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = irrot_stab_stiffness (self, geom, u)
gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    dim=size(xs,2);
    S=skewmat(zeros(dim,1));
    Vrot = repmat(S, 1, nfens); % preallocate
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            c =Ns{j}'*x;% physical location of the quadrature point
            J = x' * Nders{j};% We compute the Jacobian matrix
            Ndersp = Nders{j}/J;% derivatives wrt global coor
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (Jac<=0),error('Non-positive Jacobian');end
            for k= 1:nfens
                Vrot(:,(k-1)*dim+1:k*dim) =skewmat(Ndersp(k,:));
            end
            Ke{i} = Ke{i} + Vrot'*Vrot * (self.stabfact*Jac * w(j));
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
    
    