% Compute the nodal basis function gradients.
% Return the cell array of structures with attributes
%			   bfun_gradients{nix}.Nspd= basis function gradient matrix
%        bfun_gradients{nix}.Vpatch= nodal patch volume
%        bfun_gradients{nix}.patchconn= nodal patch connectivity
%
function bfun_gradients = nodal_bfun_gradients (self, geom)
    gcells = get(self.feblock,'gcells');
    dim= get(gcells,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts=get(integration_rule, 'npts');
    for j=1:size(pc,1)
        N{j} = bfun(gcells, pc(j,:));
        Nder{j} = bfundpar(gcells, pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    % Get the inverse map from finite element nodes to geometric cells
    gmap=get (self.f2gmap,'gcell_map');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    bfun_gradients= cell(length(gmap),1);
    for nix=1:length(gmap)
        bfun_gradients{nix}.Nspd=[];
        bfun_gradients{nix}.Vpatch=0;
        bfun_gradients{nix}.patchconn=[];
    end
    xs =gather(geom, 1:length(gmap), 'values', 'noreshape'); % coordinates of all nodes
    % Now loop over all finite element nodes
    lnmap =0*(1:length(gmap));
    patchconn = [];
    iem =1;
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=nix;
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            kconns = conns(gl,:); % connectivity
            patchconn = patch_conn (self,kconns,thisnn);
            np =length(patchconn);
            lnmap(patchconn) =1:np;% now store the local numbers
            c = xs(thisnn, :); % coordinates of central node
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);%RAW: tangent vectors?
            end
            Nspdavg=zeros(np,dim);% preallocate strain-displacement matrix
            Vpatch_dim =0; Vpatch =0;
            for k=1:length(gl)
                kconn =kconns(k,:);
                pci=find(kconn == thisnn);% at which node in the element are we?
                pci =pci(1);% in case there are repeated nodes
                if (pci<=npts)
                    px = xs(kconn, :); % coordinates of nodes
                    lx=px-ones(length(kconn),1)*c;% centered
                    if (isempty(Rm)),  xl=lx;
                    else               xl=lx*Rm;% local coordinates
                    end
                    J = Jacobian_matrix(gcells,Nder{pci},xl);
                    Jac = Jacobian_volume(gcells,kconn, N{pci}, J, xl);
                    Vpatch=Vpatch+Jac*w(pci);
                    Jac_dim = Jacobian_mdim(gcells, kconn, N{pci}, J, xl, dim);
                    Nspd = Nder{pci}*self.hadjugate(J);
                    Vpatch_dim=Vpatch_dim+Jac_dim*w(pci);
                    Nspdavg(lnmap(kconn),:)=Nspdavg(lnmap(kconn),:)+w(pci)*Nspd;
                end
            end
            if (Vpatch_dim~=0)
                Nspdavg=Nspdavg/ Vpatch_dim;
            else
                error('Zero nodal patch volume')
                Nspdavg=0*Nspdavg;
                Vpatch=0;
            end
            bfun_gradients{nix}.Nspd=Nspdavg;
            bfun_gradients{nix}.Vpatch=Vpatch;
            bfun_gradients{nix}.patchconn=patchconn;
        end
        lnmap(patchconn) = 0;
    end
end


