% Compute the material stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u1, u)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness (self, geom, u1, u, alpha)
    if (~exist('alpha','var')), alpha =1; end
    gcells = get(self.feblock,'gcells');
    ngcells =count (gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    mat = get(self, 'mater'); % Material
    matstates = get(self, 'matstates');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Kes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Kes{i} = [];
    end
		Xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
		U1s = gather(u1, 1:length(bfun_gradients), 'values', 'noreshape'); % displacement in step n+1
		x1s = Xs + U1s; % current coordinates
		eqnumss =gather(u, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = Xs(nix, :); % coordinates of central node
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);% tangent vectors?
            end
            Xnod=Xs(bfun_gradients{nix}.patchconn,:);
            U1nod=U1s(bfun_gradients{nix}.patchconn,:);
            x1nod=Xnod+U1nod;
            F1nodal= x1nod'*bfun_gradients{nix}.Nspd;
            Bnodal= self.hBlmat (self,bfun_gradients{nix}.Nspd/F1nodal,c,Rm);
            D = tangent_moduli (mat, struct ('ms',matstates{nix,1},...
                'F',F1nodal,'stiff_type','Eulerian'));
            Ke = Bnodal'* (D* (alpha * bfun_gradients{nix}.Vpatch *det(F1nodal))) *Bnodal ;
            Kes{iem} =  Ke;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
    return;
end
