% Compute the element load vectors corresponding to nonzero essential boundary
% velocities.
% Return an array of them so they may be assembled.
%    Call as
% evs = nz_ebc_loads_damping(feb, geom, v)
%     geom=geometry field
%     v=velocity field
%
function evs = nz_ebc_loads_damping(self, geom, v)
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    % Now loop over all gcells in the block
    j=1;
    for i=1:count(gcells)
        conn = conns(i,:); % connectivity
        pv = gather(v, conn, 'prescribed_values');
        if norm (pv) ~= 0
            self=set(self, 'gcells',subset(gcells,i));
            ems = damping(self, geom, v);
            mat=get(ems,'mat');
            Ke = mat{1}; % element stiffness matrix
            Fe{j} =  -Ke*pv;
            eqnums{j} = gather(v, conn, 'eqnums');
            j=j+1;
        end
    end
    Fe=Fe(1:j-1);
    eqnums=eqnums(1:j-1);
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end

