% Compute the mass matrices of the individual gcells.
%
% function ems = mass_0_antisymm (self, geom, u, ell)
%
% Return an array of them so they may be assembled.
%   Arguments
%   geom=geometry field
%   u=displacement field
%     ell= ell-th harmonic, ell=0, anti-symmetric (with respect to theta = 0) mode
%
function ems = mass_0_antisymm (self, geom, u, ell)
    if (ell~=0)
    	error (['Cannot handle ell~=0 here']);
    end
    csmultiplier=1; % cosine products give 2*pi, sine products give zero
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(u,'dim');%  This should be  3
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    mat = get(self, 'mater');% Material
    rho = get(get(mat,'property'), 'rho');
    Nexp = zeros(dim,dim*nfens);
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Me = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Me{i} =zeros(get(u,'dim')*nfens);
    end
    xs =get(geom,'values');
    for j=1:npts_per_gcell
        N = bfun(gcells,pc(j,:));
        Nder = bfundpar(gcells,pc(j,:));
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            J = Jacobian_matrix(gcells,Nder,x);
            Jac = Jacobian_volume(gcells,conn, N, J, x);
            for l = 1:nfens;
                Nexp(1:dim,(l-1)*dim+1:(l)*dim)=diag( [0,0,1] )*N(l);
            end;
            Me{i} = Me{i} + (Nexp'*Nexp) * (csmultiplier * rho * Jac * w(j));
        end
    end
    ems  = elematset(struct('mat',{Me}, 'eqnums',{eqnums}));
end
