% Compute the load-dependent stiffness matrix for the given load.
% Return an "element" matrix for assembly.
%    
% function ems = stiffness_moment (self, geom, dchi, nl)
% 
%     geom=geometry field
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.   
%     nl=nodal load. This function needs to be called only for
%           moment concentrated loads at the nodes. 
%           Note that the present routine assumes 
%           there's only one id and dir in this nodal load!
%
function ems = stiffness_moment (self, geom, dchi, nl)
    id=get(nl, 'id');
    dir=get(nl, 'dir');
    magn=get(nl, 'magn');
    kind=get(nl, 'kind');
    data=get(nl, 'data');
    if (data.lever_direction==1)||(data.lever_direction==4)
        lever_direction=[1;0;0];
    elseif (data.lever_direction==2)||(data.lever_direction==5)
        lever_direction=[0;1;0];
    elseif (data.lever_direction==3)||(data.lever_direction==6)
        lever_direction=[0;0;1];
    end
    lever_direction=lever_direction/norm(lever_direction);
    Kg={};
    eqnums={};
    if ~((dir==4)||(dir==5)||(dir==6))
        ems  = elematset(struct('mat',{Kg}, 'eqnums',{eqnums}));
        return; % nothing to be done, this is not a moment load
    end
    if (dir==4)
        force_direction= skewmat([1;0;0])*lever_direction;
    elseif (dir==5)
        force_direction= skewmat([0;1;0])*lever_direction;
    elseif (dir==6)
        force_direction= skewmat([0;0;1])*lever_direction;
    end
    if (norm(force_direction)==0)
        error('Bad lever for this moment');
    end
    force_direction=force_direction/norm(force_direction);
    if (strcmp( kind, 'quasi-tangential' ))
        e=gather(dchi,id,'eqnums');
        Kg{end+1}=1/2*magn*[force_direction*lever_direction'+lever_direction*force_direction'];
        eqnums{end+1}=e(4:6);
    end
    ems  = elematset(struct('mat',{Kg}, 'eqnums',{eqnums}));
end