% Compute the damping matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = damping(feb, geom, v)
%     geom=geometry field
%     v=velocity field
%
function ems = damping (self, geom, v)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    labels = get(gcells, 'label'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(v,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            c =Ns{j}'*x;% physical location of the quadrature point
            J = x' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                if (~isempty(labels )),  Rm =Rmh(c,J,labels(i));
                else,                    Rm =Rmh(c,J,[]);                end
            end
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt global coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (Jac<=0),error('Non-positive Jacobian');end
            B = self.hBlmat(self,Ns{j},Ndersp,c,Rm);% strain-displacement
            Dv = damping_moduli (mat, struct('xyz',c));
            Ke{i} = Ke{i} + B'*Dv*B * detJ * w_s(j);
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
