% Generate connectivity matrices of the individual nodes in the block
%    Call as
% gcells = genconn(feb)
%     x=geometry field
%     u=displacement field
%
function ngcells = genconn (self)
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    % Get the inverse map from finite element nodes to geometric cells
    gmap=get(self.f2gmap,'gcell_map');
    % Pre-allocate the gcell connectivity matrices
    conns = cell(length(gmap),1);
    iem=0;
    % Now loop over all finite element nodes in the map
    allconns = get(gcells, 'conn');
    maxnfens=0;
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=nix;
        if ~isempty(gl) % This node has a element patch in this block
            kconns = allconns(gl,:); % connectivity of all geometric cells of the patch
            patchconn  = patch_conn (self,kconns,thisnn);
            iem=iem+1;
            conns{iem} = patchconn;
            maxnfens =max( [maxnfens, length(patchconn)]);
        end
    end
    conn =zeros(length(conns),maxnfens,'int32');
    for k=1:size(conn,1)
        c=conns{k};
        conn(k,1:length(c)) =c;
        conn(k,length(c)+1:end) =c(end);
    end 
    ngcells = gcellset_N (struct('nfens',maxnfens,'conn',conn));
end


