%  Draw graphic representation for material states in all gcells.
%
% function draw_integration_points(self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with typically mandatory fields
%    x=reference geometry field
%    theta=temperature field
% with optional fields
%    scale = scale the representation of the material state graphic by this
%       factor
% Any other fields in the context struct may be interpreted by the
% implementation of the material: see the draw() function for the
% particular material.
%
function draw_integration_points(self, gv, context)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(context.x,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts'); % number of integration point
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Now loop over all gcells in the block
    context.update_context= [];
    conns = get(gcells, 'conn'); % connectivity
    for i=1:ngcells
        conn =conns(i,:);
        X = gather(context.x, conn, 'values', 'noreshape'); % reference coordinates
        T = gather(context.theta, conn, 'values', 'noreshape'); % displacement
        % Loop over all integration points
        for j=1:npts_per_gcell
            N = bfun(gcells, pc(j,:));
            Nder = bfundpar (gcells, pc(j,:));
            J = X' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_volume(gcells,conn, N, J, X);
            if (isempty(Rm))
                Ndersp = Nder/J;% derivatives wrt global coor
            else
                Ndersp = Nder/(Rm'*J);% derivatives wrt local coor
            end
            context.ms=matstates{i,j};
            context.xyz=N'*X;
            context.update_context.gradtheta = T'* Ndersp;
            context.Rm=Rm;
            draw(mat, gv, context);
        end
    end
    return;
end
