% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    tol=100*eps;
    gamma=1; beta=1/2;% the Newmark parameters: these are values determined as optimal by SW
    alph=min(beta/gamma,1);
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    % calculate effective loads at t_n-1
    tn1 = feval(torque, t, Rn1);
    if (isempty(self.Alpha))
        Alphan1=I\(Rn1'*tn1 - skewmat(Omegan1)*I*Omegan1);
        self.Alpha=Alphan1;
    end
    Alphan1=self.Alpha;
    for i=1:nsteps
        Omegan=Omegan1;
        i=0; maxi=150;
        pTheta=0*Omegan1;
        while (1)
            Theta=beta/gamma*dt*(Omegan+Omegan1-(2-gamma/beta)*(Omegan1+dt/2*Alphan1));
            DRn=rotmat(Theta);
            Rn=Rn1*DRn;
            Ra=Rn1*rotmat(alph*Theta);
            ta=feval(torque, t+alph*dt, Ra);
            Omegan=I\(DRn'*I*Omegan1+dt*Rn'*ta);
            i=i+1;
            if (i > maxi)
                if (max(abs(Theta-pTheta)) > 1e-9)
                    warning(['c1 - Failed to converge: ||residual||=' num2str(norm(Theta-pTheta))]);
                end
                break;
            end
            if (norm(Theta-pTheta) < tol)
                break;
            end
            pTheta=Theta;
        end
        % swap variables for next step
        Rn1=Rn;
        self.Alpha = (Omegan-Omegan1)/dt/gamma+(1-1/gamma)*Alphan1;
        Omegan1=Omegan;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = Omegan1;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end
