% Compute an optimal numbering of the finite element nodes using an
% appropriate method. The numbering may  be then passed to
% renum_mesh_renumber() to renumber the mesh.
% 
% function numbering=renum_mesh_numbering(fens, gcells, method)
% 
% fens, gcells = arrays of finite element nodes and geometric cells
% method =  either 'symrcm' or 'symamd' (optional argument, default
%           'symrcm') 
%     
function numbering=renumbering(self, method)
    if (~exist('method','var'))
        method ='symrcm';
    end
    
    if (strcmp(method,'symrcm'))
        % Reverse Cuthill-McKee
        p=symrcm(self.mat);
    elseif (strcmp(method,'symamd'))
        p=symamd(self.mat);
    else
        eval(['p=' method '(self.mat);']);
    end
    
    ip=(1:length(p));
    ip(p)=(1:length(p));
    numbering.p =p;
    numbering.ip =ip;
end


