%% Generate mesh along a curve.
%


%% 
% This model uses the corotational nonlinear beam element.

% Parameters:
E=10000;
nu=0.3;
L= 42; % Length of the beam
R = 1/2; % radius


%%
% Mesh
n=20;
tolerance=L/n/100;
Parameters=struct ('A',pi*R^2,'I2',pi*R^4/4,'I3',pi*R^4/4,'J',pi*R^4/2,...
    'x1x2_vector',[0 1 0], 'label',1);
% These are the points that define the spline curve.
X=[0,0,0;
    0,L/4,L*1/4;
    L/4,L/4,L*2/4;
    L/4,0,L*3/4;
    0,0,L]
[fens,gcells] = Beam_member(X,n,@gcellset_beam3,Parameters);

prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
% Initialize configuration variables
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
u=0*clone(geom,'transl');
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));
rot=0*clone(geom,'rotat');
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

u=slice(dchi,(1:3),'u');
rot=slice(dchi,(4:6),'rot');

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=20;
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on