% Boundary conditions:


if (ell~=0)
    if (  symmetric )
        % Symmetric ell-harmonic constraints
        ebc_fenids=fenode_select(fens,struct( 'box', [0,0,-inf,inf], 'inflate', tolerance));
        ebc_prescribed=ones(1,length (ebc_fenids));
        ebc_val=ebc_fenids*0;
        ebc_comp=1;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_comp=2;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);  
    else
        % Anti-symmetric ell-harmonic constraints
        ebc_fenids=fenode_select(fens,struct( 'box', [0,0,-inf,inf], 'inflate', tolerance));
        ebc_prescribed=ones(1,length (ebc_fenids));
        ebc_val=ebc_fenids*0;
        ebc_comp=2;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_comp=3;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);    
    end
else
    if (  symmetric )
        % Symmetric 0-harmonic constraints
        ebc_fenids=1:count(fens);
        ebc_prescribed=ones(1,length (ebc_fenids));
        ebc_comp=3;
        ebc_val=ebc_fenids*0;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    else
        % Anti-symmetric 0-harmonic constraints
        ebc_fenids=1:count(fens);
        ebc_prescribed=ones(1,length (ebc_fenids));
        ebc_val=ebc_fenids*0;
        ebc_comp=1;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_comp=2;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);  
    end
end

    % Apply essential boundary conditions
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);

% Assemble the system matrix
disp(['Stiffness ' num2str(get(u, 'neqns')) 'x' num2str(get(u, 'neqns')) ' assembly']);
K = start (sparse_sysmat, get(u, 'neqns'));
if (ell~=0)
    evs=stiffness(feb, geom, u, ell);
else
    if (  symmetric )
        evs=stiffness_0_symm(feb, geom, u, ell);
    else
        evs=stiffness_0_antisymm(feb, geom, u, ell);
    end
end
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
if (ell~=0)
    ems=mass(feb, geom, u, ell);
else
    if (  symmetric )
        ems=mass_0_symm(feb, geom, u, ell);
    else
        ems=mass_0_antisymm(feb, geom, u, ell);
    end
end
% If lumped mass matrix, uncomment this block
mat=get(ems,'mat');
for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    Me=mat{j};
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    mat{j}=diag(diag(Me)/dem2*em2);
end
ems=set(ems,'mat',mat);% % Consistent or lumped mass matrix
M = assemble (M, ems);
disp(['Number of DOF =' num2str(get(u, 'neqns'))]);
%

options.p= 2*neigvs;
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 2;
[W,Omega]=eigs(get(K,'mat')+OmegaShift*get(M,'mat'),get(M,'mat'),neigvs,'SM', options);
Omega =diag(diag(Omega)-OmegaShift);
[Ignore,ix] =sort (diag(Omega));


for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
end
