u=physical_units_struct;
kappa=1.81*u.W/u.K/u.M; 
rho = 2350*u.KG/u.M^3;
cv =0.22*u.J/u.K/u.KG*rho;
d=  0.13*u.M;
% Tsoil=(8+  273.15)*u.K;
% Twire=(34+  273.15)*u.K;
% Ta=(-5+  273.15)*u.K;
Tsoil=(8)*u.K;
Twire=(34)*u.K;
Ta=(-5)*u.K;
h=15*u.W/u.K/u.M^2;
tolerance=d/10000;
num_integ_pts=1; % 1-point quadrature
fens=fenodeset(struct('id',(1:6)','xyz',[0,d;0,2*d;d,2*d;d,d;0,0;d,0]));
gcells=gcellset_T3(struct('id',(1:4)','conn',[1,4,2;3,2,4;1,5,4;6,4,5]));
edge_gcells=gcellset_L2(struct('id',(1:6)','conn',[5,6;6,4;4,3;3,2;2,1;1,5]));
edge_groups={1,2,3,4,5,6};
prop = property_diffusion(struct('conductivity',kappa,'source',0));
mater=mater_diffusion(struct('property',prop));
feb = feblock_diffusion(struct ('mater',mater,...
    'gcells',gcells,...
    'integration_rule',tri_rule(num_integ_pts)));
sfeb = feblock_diffusion(struct ('mater',mater,...
    'gcells',subset(edge_gcells,edge_groups{4}),...
    'integration_rule',gauss_rule(1,2),...
    'surface_transfer', h));
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
theta=field(struct('name',['theta'], 'dim', 1, 'nfens',get(geom,'nfens')));
fenids=connected_nodes  (subset(edge_gcells,edge_groups{1}));
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+Tsoil;
theta = set_ebc(theta, fenids, prescribed, comp, val);
fenids=[intersect(connected_nodes(subset(edge_gcells,edge_groups{2})),...
connected_nodes(subset(edge_gcells,edge_groups{3})))];
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+Twire;%
theta = set_ebc(theta, fenids, prescribed, comp, val);
theta = apply_ebc (theta);
theta = numbereqns (theta);
amb = clone(theta, ['amb']);
fenids=connected_nodes  (subset(edge_gcells,edge_groups{4}));
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+Ta;
amb = set_ebc(amb, fenids, prescribed, comp, val);
amb = apply_ebc (amb);


K = start (sparse_sysmat, get(theta, 'neqns'));
K = assemble (K, conductivity(feb, geom, theta));
K = assemble (K, surface_transfer(sfeb, geom, theta));
F = start (sysvec, get(theta, 'neqns'));
F = assemble (F, nz_ebc_loads_conductivity(feb, geom, theta));
F = assemble (F, surface_transfer_loads(sfeb, geom, theta, amb));
theta = scatter_sysvec(theta, K\F);

% Plothot
gv=graphic_viewer;
gv=reset (gv,[]);
T=get(theta,'values');
XY=get(geom,'values');
conn= get(gcells,'conn');
draw(gcells, gv, struct ('x',geom, 'u',0*geom, 'facecolor','none','edgecolor',[0.49, 0.49, 0.49]));
dcm=data_colormap(struct ('range',[min(T),max(T)], 'colormap',jet));
for  isovalue=linspace(min(T),max(T),20)
    draw_isosurface(feb,gv, struct ('x', geom,'u',0*geom,'scalarfield',theta,'isovalue',isovalue,'color',map_data(dcm, isovalue)));
end
draw_integration_points(feb, gv, struct ('x',geom,'u',0*geom, ...
    'theta', theta, 'scale', 00.000004,'color','red'));
draw_colorbar(gv, struct('colormap',get(dcm,'colormap'),...
                'position', [0.8, 0.1, 0.05, 0.6],...
                'minmax',get(dcm,'minmax'),'label','Temperature'));
view(2)
set_graphics_defaults
labels('x','y')