u=physical_units_struct;
kappa=[0.06*u.W/u.K/u.M]; % conductivity matrix
volume_fraction= 0.06;
cm = 700*u.J/u.K/u.KG;% specific heat per unit mass
rho=2650*u.KG/u.M^3;% mass density
cv =volume_fraction*cm*rho;% specific heat per unit volume
h= 50*u.W/u.K/u.M^2;
Q=0; %  heat source is absent
Tamb=(273+20)*u.K;
Tini=(273+2000)*u.K;
L=0.06*u.M;% thickness
dt=0.1*u.SEC; % time step
tend= 70; % length of the time interval
t=0;
theta = 1; % generalized trapezoidal method
online_graphics= true;% plot the solution as it is computed?
figure;
n=4*50;% needs to be multiple of five
[fens,gcells] = L2_block(L,n,1.0); % Mesh
prop=property_diffusion(struct('conductivity',kappa,...
    'specific_heat',cv,'rho',rho,'source',Q));
mater=mater_diffusion (struct('property',prop));
feb = feblock_diffusion (struct (...
    'mater',mater,...
    'gcells',gcells,...
    'integration_rule',simpson_1_3_rule));
point_gcells= [gcellset_P1(struct('id',[1;n+1],'conn',[1;n+1]))];
pointfeb = feblock_diffusion (struct (...
    'mater',mater,...
    'gcells', point_gcells,...
    'integration_rule',point_rule,...
    'surface_transfer', h));
geom = field(struct ('name',['geom'], 'dim', 1, 'fens',fens));
tempn = field(struct ('name',['temp'], 'dim', 1,...
    'nfens',get(geom,'nfens')));
tempn = numbereqns (tempn);
tempn = scatter_sysvec(tempn,gather_sysvec(tempn)*0+Tini);
amb = clone(tempn, ['amb']);
conn = connected_nodes(point_gcells);
amb = set_ebc(amb, conn, conn*0+1, [], conn*0+Tamb);
amb = apply_ebc (amb);
K = start (sparse_sysmat, get(tempn, 'neqns'));
K = assemble (K, conductivity(feb, geom, tempn));
K = assemble (K, surface_transfer(pointfeb, geom, tempn));
Km = get(K,'mat');
C = start (sparse_sysmat, get(tempn, 'neqns'));
C = assemble (C, capacity(feb, geom, tempn));
Cm = get(C,'mat');
while t<tend+0.1*dt % Time stepping
    if online_graphics
        plot([0, 0],[0,Tini-273],'.'); hold on
        plot(get(geom,'values'),get(tempn,'values')-273,'linewidth', 2*(1- 0.9999*t/tend));
        labels(' Thickness [m]','  Temperature [degrees Celsius]')
        figure(gcf);
        title (['Time =' num2str(t)]); pause(0.0001); hold off; 
    end
    tempn1 = tempn;
    F = start (sysvec, get(tempn, 'neqns'));
    F = assemble(F, surface_transfer_loads(pointfeb,geom,tempn,amb));
    Tn=gather_sysvec(tempn);
    Tn1 = (1/dt*Cm+theta*Km) \ ((1/dt*Cm-(1-theta)*Km)*Tn+...
        get(F,'vec'));
    tempn = scatter_sysvec(tempn1,Tn1);
    t=t+dt;
end
