%%  Define parameters
% clear all classes;
E=210000;% MPa
nu=0.3;
W=0.5;% millimeter
H=10;% millimeter
L= 100;
rho=8e-9; %
htol=min([L,H,W])/1000;
neigvs = 12;
Reference= 1e4*[0.08283,0.49558,1.28596,1.30131,2.35724,3.5852,3.85257,4.92538,6.33902,6.40234,7.80019,8.9205];

graphics =~~true;
annloc=[L/2, 0, 3.5*H];
    
% Choose mesh resolution
clear mesd
mix = 1;
mesd(mix).mult=20;
mesd(mix).ns=1;

% Choose the element formulation
clear eltyd
eix=1;

% % NICE tetrahedron
% eltyd(eix).mf =@T4_block;
% eltyd(eix).blf =@feblock_defor_ss_nice;
% eltyd(eix).integration_rule= simplex_nq_rule (3);
% eltyd(eix).surface_integration_rule=tri_rule(1);
% eltyd(eix).styl='r*-';


% NICE  hexahedron
% eltyd(eix).mf =@H8_block;
% eltyd(eix).blf =@feblock_defor_ss_nice;
% eltyd(eix).integration_rule= tensprod_nq_rule (3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='cd-';
%

% NICE  27-node hexahedron
% eltyd(eix).mf =@H27_block;
% eltyd(eix).blf =@feblock_defor_ss_nice;
% eltyd(eix).integration_rule=tensprod_nq_rule (struct('dim',3, 'order',2 ));
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='mx--';


% Selective reduced integration hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss_sri;
% eltyd(eix).integration_rule=gauss_rule (3,[1,2]);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='md-';
%

% Full integration hexahedron
eltyd(eix).mf =@H8_block;
eltyd(eix).blf =@feblock_defor_ss;
eltyd(eix).integration_rule=gauss_rule (3,2);
eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
eltyd(eix).styl='mx--';
%

% Full integration 27-node hexahedron
% eltyd(eix).mf =@H27_block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='mx--';
%

% Reduced integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,2);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='ro-';
%


% eltyd(eix).label ='H8';
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule= gauss_rule (3, 2);
% eltyd(eix).surface_integration_rule=tensprod_nq_rule (struct('dim',2,'order',1));
% eltyd(eix).styl='mp-';
%



% Full integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='b*-';
%

% T10 tetrahedron
% eltyd(eix).mf =@T10_block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=tet_rule (4);
% eltyd(eix).surface_integration_rule=tri_rule(3);
% eltyd(eix).styl='gd-';
%


eix = 1;
mix = 1;
ns=[];
uzs=[];

n=mesd(mix).ns;
mult =mesd(mix).mult;
%% Create the mesh and initialize the geometry
[fens,gcells]= feval (eltyd(eix).mf, W, L, H,1,mult*n,2*n);
%                    mesh{1}=fens;
%                     mesh{2}=gcells;
%                     drawmesh(mesh,'shrink', 0.8,'facecolor','red');
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));

feb = feval (eltyd(eix).blf, struct ('mater',mater, 'gcells',gcells, 'integration_rule',eltyd(eix).integration_rule));
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));

%% Define the displacement field, and zero it out
u   = 0*geom;

%% Apply the EBC''s
ebc_fenids=fenode_select(fens,struct('box',[0 W 0 0 0 H]));
ebc_prescribed=ones(length(ebc_fenids),1);
ebc_comp=[];
ebc_val=zeros(length(ebc_fenids),1);
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens,struct('box',[W W 0 L 0 H]));
ebc_prescribed=ones(length(ebc_fenids),1);
ebc_comp=ebc_fenids*0+1;
ebc_val=zeros(length(ebc_fenids),1);
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

%% Number the equations
u   = numbereqns (u);
disp (['n=' num2str(n) ', number of equations =' num2str(get(u,'neqns'))])

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=stiffness(feb, geom, u);
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
% mat=get(ems,'mat');
% for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
%     Me=mat{j};
%     em2=sum(sum(Me));
%     dem2=sum(diag(Me));
%     mat{j}=diag(diag(Me)/dem2*em2);
% end
% ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
M = assemble (M, ems);
disp(['Number of DOF =' num2str(get(u, 'neqns'))]);
%
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 2;
mK=get(K,'mat'); mK=(mK+mK')/2;
mM=get(M,'mat'); mM=(mM+mM')/2;
[W,Omega]=eigs(mK,mM,neigvs,'SM', options);
Omega =diag(diag(Omega));
[Ignore,ix] =sort (diag(Omega));


for i=1:neigvs
    f=sqrt(Omega(ix(i),ix(i)))/2/pi;
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(f) ' [Hz]' ', frequency error ' num2str(abs(f-Reference(i))/Reference(i)*100) ' [%]']);
end


% Plot
if graphics
    Camera =[ 903.8644   18.0322    7.4626    9.0142   41.4431    7.4725         0         0    1.0000    4.8929];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=65;
    w=clone(u,'w'); % make a copy of u
    for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
        %         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,[]);
            camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18));
            draw_axes (gv,struct('length',20 ));
            pause(0.005);
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(0.5); % next eigenvector
        gv=reset (gv,[]);
        camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.02));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',20 ));
        pause(0.55);
        
    end
end
