function space
disp('LE1 NAFEMS benchmark, 3-D meshes');
pu=physical_units_struct;
% Parameters:
E = 210e3*pu.MEGA*pu.PA;% 210e3 MPa
nu = 0.3;
p = 10*pu.MEGA*pu.PA;% 10 MPA Outward pressure on the outside ellipse
A=3.25*pu.M; B= 2.75*pu.M;
C=2.00*pu.M; D= 1.00*pu.M;
thickness =0.1*pu.M;
tolerance =0.00001*pu.M;
sigma_yD= 92.7*pu.MEGA*pu.PA;% tensile stress at [2.0, 0.0] meters
n=4;% number of elements through the thickness

% % NICE tetrahedron
Meshf =@T4_block;
Convertf=[];
febf =@feblock_defor_ss_nice;
Volume_integration_rule= simplex_nq_rule (3);
Edge_integration_rule=simplex_nq_rule (2);
Style='r*-';


% NICE  hexahedron
% eltyd(eix).mf =@H8_block;
% eltyd(eix).blf =@feblock_defor_ss_nice;
% eltyd(eix).integration_rule= tensprod_nq_rule (3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='cd-';
%

% NICE  27-node hexahedron
Meshf =@H27_block;
Convertf=[];
febf =@feblock_defor_ss_nice;
Volume_integration_rule=tensprod_nq_rule (struct('dim',3, 'order',2 ));
Edge_integration_rule=tensprod_nq_rule (struct('dim',2, 'order',2 ));
Style='mx--';


% Selective reduced integration hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss_sri;
% eltyd(eix).integration_rule=gauss_rule (3,[1,2]);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='md-';
%

% Full integration hexahedron
% eltyd(eix).mf =@H8_block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,2);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='mx--';
%

% Full integration 27-node hexahedron
Meshf =@H27_block;
Convertf=[];
febf =@feblock_defor_ss;
Volume_integration_rule=gauss_rule (3,3);
Edge_integration_rule=gauss_rule(2, 3);
Style='mx--';
%

% Reduced integration H20
% Meshf =@H20_block;
% Convertf=[];
% febf =@feblock_defor_ss;
% Volume_integration_rule=gauss_rule (3,2);
% Edge_integration_rule=gauss_rule(2, 2);
% Style='ro-';
%


% eltyd(eix).label ='H8';
Meshf =@H8_block;
Convertf=[];
febf =@feblock_defor_ss;
Volume_integration_rule=gauss_rule (3,2);
Edge_integration_rule=gauss_rule(2,2);
% eltyd(eix).styl='mp-';
%



% Full integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='b*-';
%

% T10 tetrahedron
% eltyd(eix).mf =@T10_block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=tet_rule (4);
% eltyd(eix).surface_integration_rule=tri_rule(3);
Meshf =@T10_block;
Convertf=[];
febf =@feblock_defor_ss;
Volume_integration_rule=tet_rule (4);
Edge_integration_rule=tri_rule(3);
% eltyd(eix).styl='gd-';
%


% Mesh'
[fens,gcells]=Meshf(1.0, pi/2, thickness, n, n*2, 1);
if (~isempty( Convertf ))
  [fens,gcells] = Convertf(fens,gcells);
end
bdry_gcells = mesh_bdry(gcells, []);
icl = gcell_select(fens, bdry_gcells, struct('box', [D,D,0,pi/2, 0,thickness],'inflate',1/1000/n));
xy=get (fens,'xyz');
for i=1:count(fens)
    t=xy(i,1); a=xy(i,2);
    xy(i,:)=[(t*A+(1-t)*C)*cos(a) (t*B+(1-t)*D)*sin(a) xy(i,3)];
end
fens=set(fens,'xyz', xy);
% drawmesh( {fens,gcells})
% view (2)
% return
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = febf(struct ('mater',mater, 'gcells',gcells, 'integration_rule',Volume_integration_rule));
efeb = febf (struct ('mater',mater, 'gcells',subset(bdry_gcells,icl),...
    'integration_rule',Edge_integration_rule));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = clone(geom,'u');
u   = u*0; % zero out
% Apply EBC's
% plane of symmetry
ebc_fenids=fenode_select (fens,struct('box',[0 Inf 0 0 0 thickness],'inflate',tolerance));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_prescribed*0+2;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
% plane of symmetry
ebc_fenids=fenode_select (fens,struct('box',[0 0 0 Inf 0 thickness],'inflate',tolerance));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_prescribed*0+1;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
% plane-strain constraint
ebc_fenids=1:count(fens);
ebc_prescribed=ones(1,length(ebc_fenids));
ebc_comp=ebc_prescribed*0+3;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);
% tic
% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(feb, geom, u));
% Load
fi=force_intensity(struct('magn',@(x) (p*[B/A*x(1),A/B*x(2),0]'/norm([B/A*x(1),A/B*x(2),0]))));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, distrib_loads(efeb, geom, u, fi, 2));
% Solve
u = scatter_sysvec(u, K\F);
% get(u,'values')
% toc, tic
% Plot
gv=graphic_viewer;
gv=reset (gv,struct ('limits',[0 1.06*A 0 1.6*A 0 thickness]));
scale=1000;
cmap = jet;
fld = field_from_integration_points(feb, geom, u, [], 'Cauchy',2);
corner=fenode_select (fens,struct('box',[C C 0 0 0 thickness],'inflate',thickness/10000));
sigma_yD_approx=mean(gather (fld, corner,'values'));
disp( ['Corner stress sigma_y=' num2str(sigma_yD_approx/(pu.MEGA*pu.PA)) ' MPa'])
disp( [' i.e. ' num2str(sigma_yD_approx/sigma_yD*100) '% of the Reference value'])
nvals=get(fld,'values');%min(nvals),max(nvals)
nvalsrange=[min(nvals),max(nvals)];
dcm=data_colormap(struct ('range',nvalsrange, 'colormap',cmap));
colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
draw(feb,gv, struct ('x', geom, 'u', +scale*u,'colorfield',colorfield, 'shrink',1.0));
draw(feb,gv, struct ('x', geom, 'u', 0*u,'facecolor','none', 'shrink',1.0));
% draw(efeb,gv, struct ('x', geom, 'u', 0*u,'facecolor','red', 'shrink',1.0));
colormap(cmap);
draw_colorbar(gv,struct('position',[0.815 0.15 0.05 0.7],'minmax',nvalsrange,'label','\sigma_y'));
set_graphics_defaults
% assignin('caller','faesor_test_passed',((norm([93.029698724563474]-gather(fld, corner,'values')))<1e-9))