% Reference: C.M. Wang, W.X. Wu, C. Shu, T. Utsunomiya, LSFD method
% for accurate vibration modes and modal stress-resultants of freely
% vibrating plates that model VLFS, Computers and Structures 84 (2006)
% 23292339
function rectangular_plate

pu=physical_units_struct;
% Parameters:
E = 210e3*pu.MEGA*pu.PA;% 210e3 MPa
nu = 0.3;
rho= 7850*pu.KG/pu.M^3;
a=4.00*pu.M; b=1.00*pu.M; h= 0.1*pu.M;
% Mesh refinement
na= 13; nb=  9; nh =3;
na= 29; nb=  9; nh =3;
na= 15; nb=  4; nh =2;
na= 6; nb=  2; nh =1;
% na= 5; nb=  4; nh =2;



% Fundamental frequency
f_fundamental =[0 0 0  0 0 0 33.78, 82.28,  92.99, 170.06];
shift =(2*pi*f_fundamental(1))^2;;
graphics = ~false;




% % NICE tetrahedron
Meshf =@T4_block;
Convertf=[];
febf =@feblock_defor_ss_nice;
Volume_integration_rule= simplex_nq_rule (3);
Edge_integration_rule=simplex_nq_rule (2);
Stabfactor= 0.0005;
Style='r*-';


% T10 tetrahedron
Meshf =@T10_block;
Convertf=[];
febf =@feblock_defor_ss;
Volume_integration_rule=tet_rule (4);
Edge_integration_rule=tri_rule(3);
Style='gd-';
%

% NICE  hexahedron
% eltyd(eix).mf =@H8_block;
% eltyd(eix).blf =@feblock_defor_ss_nice;
% eltyd(eix).integration_rule= tensprod_nq_rule (3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='cd-';
%

% NICE  27-node hexahedron
% Meshf =@H27_block;
% Convertf=[];
% febf =@feblock_defor_ss_nice;
% Volume_integration_rule=tensprod_nq_rule (struct('dim',3, 'order',2 ));
% Edge_integration_rule=tensprod_nq_rule (struct('dim',2, 'order',2 ));
% Style='mx--';


% Selective reduced integration hexahedron
% eltyd(eix).mf =@block;
% eltyd(eix).blf =@feblock_defor_ss_sri;
% eltyd(eix).integration_rule=gauss_rule (3,[1,2]);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='md-';
%

% Full integration hexahedron
% eltyd(eix).mf =@H8_block;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,2);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='mx--';
%

% Full integration 27-node hexahedron
% Meshf =@H27_block;
% Convertf=[];
% febf =@feblock_defor_ss;
% Volume_integration_rule=gauss_rule (3,3);
% Edge_integration_rule=gauss_rule(2, 3);
% Style='mx--';
%

% Reduced integration H20
% Meshf =@H20_block;
% Convertf=[];
% febf =@feblock_defor_ss;
% Volume_integration_rule=gauss_rule (3,2);
% Edge_integration_rule=gauss_rule(2, 2);
% Style='ro-';
%


% eltyd(eix).label ='H8';
% Meshf =@H8_block;
% Convertf=[];
% febf =@feblock_defor_ss;
% Volume_integration_rule=gauss_rule (3,2);
% Edge_integration_rule=gauss_rule(2,2);
% Style='mp-';
%



% Full integration H20
% eltyd(eix).mf =@blockH20;
% eltyd(eix).blf =@feblock_defor_ss;
% eltyd(eix).integration_rule=gauss_rule (3,3);
% eltyd(eix).surface_integration_rule=gauss_rule(2, 2);
% eltyd(eix).styl='b*-';
%


% Mesh'

% Mesh
[fens,gcells] = Meshf(a,b,h,na,nb,nh);;

if (~isempty( Convertf ))
  [fens,gcells] = Convertf(fens,gcells);
end
disp(['#fens=' num2str(count(fens)) ', ' '#gcells=' num2str(count(gcells))])

% drawmesh( {fens,gcells})
% view (2)
% return
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));

feb = febf(struct ('mater',mater, 'gcells',gcells, 'integration_rule',Volume_integration_rule,'stabfact',Stabfactor));



% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out

% %  Simple  support EBC
% nl0=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
%     'inflate',0.001*h))];
% nl1=[fenode_select(fens, struct('box', [a,a,-Inf,Inf,-Inf,Inf],...
%     'inflate',0.001*h))];
% nw0=[fenode_select(fens, struct('box', [0,a,0,0,-Inf,Inf],...
%     'inflate',0.001*h))];
% nw1=[fenode_select(fens, struct('box', [0,a,b,b,-Inf,Inf],...
%     'inflate',0.001*h))];
%
% % Suppress transverse displacements
% u   = set_ebc(u,nl0,nl0*0+1, [3], nl0*0);
% u   = set_ebc(u,nl1,nl1*0+1, [3], nl1*0);
% u   = set_ebc(u,nw0,nw0*0+1, [3], nw0*0);
% u   = set_ebc(u,nw1,nw1*0+1, [3], nw1*0);
%
% % Suppress tangential displacements along the edges
% u   = set_ebc(u,nl0,nl0*0+1, [2], nl0*0);
% u   = set_ebc(u,nl1,nl1*0+1, [2], nl1*0);
% u   = set_ebc(u,nw0,nw0*0+1, [1], nw0*0);
% u   = set_ebc(u,nw1,nw1*0+1, [1], nw1*0);

u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=stiffness(feb, geom, u);
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
% mat=get(ems,'mat');
% for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
%     Me=mat{j};
%     em2=sum(sum(Me));
%     dem2=sum(diag(Me));
%     mat{j}=diag(diag(Me)/dem2*em2);
% end
% ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
M = assemble (M, ems);

disp(['Number of DOF =' num2str(get(u, 'neqns'))]);

%
neigvs = 20;
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 0;
[W,Omega]=eigs(get(K,'mat')+ shift*get(M,'mat'),get(M,'mat'),neigvs,'SM', options);
Omega =diag(diag(Omega-shift));
[Ignore,ix] =sort (diag(Omega));



for i=7:10
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
    disp(['             normalized frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi/f_fundamental(i)*100) ' [%]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [ -0.5986   -1.2414    0.8919    0.0992    0.0992    0.0193         0         0    0.0010    0.0068];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[mean(box(1:2)), mean(box(3:4))-mean(box(1:2)), mean(box(5:6))-1.2*h];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=3;
    w=clone(u,'w'); % make a copy of u
    
    for i=6+(1:2)
        f=sqrt(Omega(ix(i),ix(i)))/2/pi;
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(f) ' [Hz]']);
        %         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        frame=1;
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,struct('limits', inflate_box(box,scale*max(wmag))));
            %                         camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18,'backgroundcolor','w'));
            draw_axes (gv,struct('length',a/5 ));
            pause(0.005);
            gif_animation_add_frame(gcf,frame,[mfilename '-m-' num2str(i) '.gif'],30)
            frame=frame+1;
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
        %         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',a/5 ));
        pause(2.55);
        
    end
end

end