function KS_sandwich_vibration_ss_again

% Material: The data from the above cited article. The ratio of the Young's
% moduli is variable, and the number of layers also varies.
angles =[0 90 0 90 0];

% Compute  all the other parameters
E1=131000; E2=10340; E3=10340; G12=6895;  G13=6205; G23=6895;
nu12= 0.22; nu13= 0.22; nu23= 0.49; rho = 1.627e-9;
L=200; W=200;
%------------------------skin-------------------------
props = property_linel_ortho (...
    struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23,'rho',rho)); %% 1627 kg/m^3
maters = mater_defor_ss_linel_triax (struct('property',props));
%-------------------------core------------------------------------------------
E1=6.89; E2=6.89; E3=6.89; G12=E1/2;  G13=E1/2; G23=E1/2;
nu12= 0; nu13= 0; nu23= 0; rho = 0.097e-9;
propc = property_linel_ortho (...
    struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23,'rho',rho));
materc = mater_defor_ss_linel_triax (struct('property',propc));


nLayers =length(angles);
[nL,nW] =adeal(6*[1,1]);
nts= 2*ones(length(angles),1);% number of elements per layer
% ts= t/length(angles)*ones(length(angles),1);% layer thicknesses only for even thickness
ts = [5,5,20,5,5];
t=sum(ts);
graphics = false;

    function Rm = LayerRm(XYZ, ts, label)% label equals the layer number here
        Rm= rotmat(angles(label)/180*pi* [0,0,1]);
    end
% Mesh
[fens,gcells] = H8_composite_plate(L,W,ts,nL,nW,nts);
[fens,gcells] = H8_to_H20(fens,gcells);
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 1)))},'gcells', 'facecolor','r');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 2)))},'gv',gv,'gcells', 'facecolor','g');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 3)))},'gv',gv,'gcells', 'facecolor','k');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 4)))},'gv',gv,'gcells', 'facecolor','m');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 5)))},'gv',gv,'gcells', 'facecolor','c');

feb = feblock (struct ('gcells',gcells));
C = start (sparse_sysmat, count(fens));
C = assemble (C, connection(feb)); clear feb
numbering=renumbering(C, 'symamd'); clear C
[fens, gcells]=mesh_renumber(numbering,fens,gcells);

% Finite element block
gls=[gcell_select(fens,gcells,struct('label',1)),gcell_select(fens,gcells,struct('label',2)),gcell_select(fens,gcells,struct('label',4)),gcell_select(fens,gcells,struct('label',5))];
glc=gcell_select(fens,gcells,struct('label',3));

febs = feblock_defor_ss (struct ('mater',maters,...
    'gcells',subset(gcells,gls), ...
    'integration_rule',gauss_rule (3,2),'Rm',@LayerRm));

febc = feblock_defor_ss (struct ('mater',materc,...
    'gcells',subset(gcells,glc), ...
    'integration_rule',gauss_rule (3,2),'Rm',@LayerRm));


OmegaShift=(2*pi*3) ^ 2;

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out

%  Simple  support EBC
nl0=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
    'inflate',0.001*t))];
nl1=[fenode_select(fens, struct('box', [L,L,-Inf,Inf,-Inf,Inf],...
    'inflate',0.001*t))];
nw0=[fenode_select(fens, struct('box', [0,L,0,0,-Inf,Inf],...
    'inflate',0.001*t))];
nw1=[fenode_select(fens, struct('box', [0,L,W,W,-Inf,Inf],...
    'inflate',0.001*t))];

% Suppress transverse displacements
u   = set_ebc(u,nl0,nl0*0+1, [3], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [3], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [3], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [3], nw1*0);

% Suppress tangential displacements along the edges  
u   = set_ebc(u,nl0,nl0*0+1, [2], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [2], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [1], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [1], nw1*0);



u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, cat(2,stiffness(febs, geom, u),stiffness(febc, geom, u)));
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(febs, geom, u);
% If lumped mass matrix, uncomment this block
% mat=get(ems,'mat');
% for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
%     Me=mat{j};
%     em2=sum(sum(Me));
%     dem2=sum(diag(Me));
%     mat{j}=diag(diag(Me)/dem2*em2);
% end
% ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
emc=mass(febc, geom, u);
M = assemble (M, cat(2,ems,emc));
% M=assemble(M,ems);

disp(['Number of DOF =' num2str(get(u, 'neqns'))]);

%
neigvs = 8;
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 0;

A=get(K,'mat')+OmegaShift*get(M,'mat');
% nnz(A), figure;spy(A)
A = chol(A,'lower');
% nnz(A), figure;spy(A)
% x = ;A'\(A\b)

tic;
[W,Omega]=eigs(@(b)cs_ltsolve(A, cs_lsolve(A, b)),get(u, 'neqns'),get(M,'mat'),neigvs,'SM', options);
% [W,Omega]=eigs(@(b)A'\(A\b),get(u, 'neqns'),get(M,'mat'),neigvs,'SM', options);
toc
Omega =diag(diag(Omega)-OmegaShift);
[Ignore,ix] =sort (diag(Omega));



for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [   -1.0612
        -0.8377
        1.0948
        -0.0775
        0.0025
        0.0093
        0
        0
        0.0010
        0.0033];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[mean(box(1:2)), mean(box(3:4))-mean(box(1:2)), mean(box(5:6))];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=1365;
    w=clone(u,'w'); % make a copy of u
    
    for i=1:6
        f=sqrt(Omega(ix(i),ix(i)))/2/pi;
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(f) ' [Hz]']);
        %         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,[]);
            %             camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18));
            draw_axes (gv,struct('length',20 ));
            pause(0.005);
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
        %         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',20 ));
        pause(2.55);
        
    end
end

end