
function sandwich_plate
    % Skin material:
    %     Layerwise Composite Plate Analysis Using an Element Template Methodology
    % J. Engrg. Mech. Volume 124, Issue 5, pp. 587-590 (May 1998)
    %
    E1=181000; E2=10300; E3=10300; G12=7170;  G13=7170; G23=2870;
    nu12= 0.28; nu13= 0.28; nu23= 0.33;
    props = property_linel_ortho (...
        struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23));
    %    Sandwich core properties
    E1=800; E2=130; E3=130; G12=417;  G13=417; G23=270;
    nu12= 0.28; nu13= 0.28; nu23= 0.33;
    propc = property_linel_ortho (...
        struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23));
    
    L=200; W=400;
    ts=[1,12,1]*1;
    t=sum(ts);
    magn = -0.015;
    angles =[90,0,90];
    nLayerss =2;%skin
    nLayersc =3;%core
    [nL,nW] =adeal([8,8]);
    graphics = ~false;
    % This function returns the layer in which the quadrature point XYZ is located.
    function l = Layer(XYZ, tans)
        if (XYZ(3)<ts(3))
            l=1;
        else
            if (XYZ(3)>ts(3) && XYZ(3)<sum(ts)-ts(1))
                l=2;
            else
                l=3;
            end
        end
    end
    %    Compute the  material rotation matrix at the quadrature point XYZ.
    function Rm = LayerRm(XYZ, tans, label)
        Rm= rotmat(angles(Layer(XYZ, tans))/180*pi* [0,0,1]);
    end
    % Mesh
    [fens1,gcells1] = H8_block(L,W,ts(1),nL,nW,nLayerss);
    gcells1=set(gcells1,'label',1);
    [fens1] = translate_mesh(fens1, [0,0,sum(ts(1:2))]);
    [fens2,gcells2] = H8_block(L,W,ts(2),nL,nW,nLayersc);
    gcells2=set(gcells2,'label',2);
    [fens2] = translate_mesh(fens2, [0,0,sum(ts(1:1))]);
    [fens3,gcells3] = H8_block(L,W,ts(3),nL,nW,nLayerss);
    gcells3=set(gcells3,'label',3);
    tol=min(ts)*0.001;
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tol);
    gcells=cat(gcells1,gcells2);
    [fens,gcells,gcells3] = merge_meshes(fens, gcells, fens3, gcells3, tol);
    gcells=cat(gcells,gcells3);
    %gv=drawmesh({fens,subset(gcells,gcell_select(fens,gcells,struct('label',1)))},'gcells','facecolor','r');
    %gv=drawmesh({fens,subset(gcells,gcell_select(fens,gcells,struct('label',2)))},'gv',gv,'gcells','facecolor','g');
    %gv=drawmesh({fens,subset(gcells,gcell_select(fens,gcells,struct('label',3)))},'gv',gv,'gcells','facecolor','b');
    
    
    [fens,gcells] = H8_to_H20(fens,gcells);
    
    % Material
    maters = mater_defor_ss_linel_triax (struct('property',props));
    materc = mater_defor_ss_linel_triax (struct('property',propc));
    % Finite element block
    gls=[gcell_select(fens,gcells,struct('label',1)),gcell_select(fens,gcells,struct('label',3))];
    glc=gcell_select(fens,gcells,struct('label',2));
    febs = feblock_defor_ss (struct ('mater',maters,...
        'gcells',subset(gcells,gls), ...
        'integration_rule',gauss_rule(3,2),'Rm',@LayerRm));
    febc = feblock_defor_ss (struct ('mater',materc,...
        'gcells',subset(gcells,glc), ...
        'integration_rule',gauss_rule(3,2),'Rm',@LayerRm));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    % Define the displacement field
    u   = 0*geom; % zero out
    % Apply EBC's
    nl=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
        'inflate',tol))];
    u   = set_ebc(u,nl,nl*0+1, [], nl*0);
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    % Assemble the system matrix
    K = start (sparse_sysmat, get(u, 'neqns'));
    K = assemble (K, cat(2,stiffness(febs, geom, u),stiffness(febc, geom, u)));
    %
    fi= force_intensity(struct ('magn',[0;0; magn]));
    bdry_gcells = mesh_bdry(gcells, []);
    bcl = gcell_select(fens, bdry_gcells, ...
        struct ('box',[-Inf,Inf,-Inf,Inf,t,t],'inflate',tol));
    lfeb = feblock_defor_ss(struct ('mater',maters, 'gcells',subset(bdry_gcells,bcl),...
        'integration_rule',gauss_rule(2,2)));
    F = start (sysvec, get(u, 'neqns'));
    F = assemble (F, distrib_loads(lfeb, geom, u, fi,2));
    u = scatter_sysvec(u, K\F);
    
    % Plot
    if graphics
        gv=graphic_viewer;
        gv=reset (gv,[]);
        scale=50;
        % draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','none'));
        fld = field_from_integration_points(febs, geom, u, [], 'Cauchy', 1);
        nvals=get(fld,'values');
        dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        draw(febs, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
        fld = field_from_integration_points(febc, geom, u, [], 'Cauchy', 1);
        nvals=get(fld,'values');
        dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        draw(febc, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
        labels(  []);;
        colormap(jet);
        cbh=colorbar;
        set(cbh,...
            'Position',[0.8 0.15 0.015 0.7],...
            'YLim',[0,1],...
            'YTick',[0,1],...
            'YTickLabel',{[num2str(min(nvals))],[num2str(max(nvals))]});%[num2str((min(nvals)))],[num2str((max(nvals)))]
        set(get(cbh,'XLabel'),'String','\sigma_{y}');
        title ('Stress');
        set_graphics_defaults 
    end
end
