                                                                     
                                                                     
                                                                     
                                             
function testcase
clc
% Material:
%     Layerwise Composite Plate Analysis Using an Element Template Methodology
    % J. Engrg. Mech. Volume 124, Issue 5, pp. 587-590 (May 1998)
%  E1=181000; E2=10300; E3=10300; G12=7170;  G13=7170; G23=2870;% MPa
 E1=181000; E2=E1/40; E3=E1/40; G12=0.6*E2;  G13=0.6*E2; G23=0.5*E2;% MPa
    nu12= 0.25; nu13= 0.25; nu23= 0.25;
    L=200; W=200; t=L/100;% mm
    magn = -0.015;
    angles =[0 90 90 0];
    nLayers =length(angles);
    [nL,nW,nt] =adeal([8,8,2*nLayers]);
    graphics = ~false;

    function l = Layer(XYZ, ts)
        for l=1:nLayers
            if (XYZ(3)<l*t/nLayers) &&  (XYZ(3)>(l-1)*t/nLayers)
                break;
            end
        end
%         l
    end
    function Rm = LayerRm(XYZ, ts, label)
        Rm= rotmat(angles(Layer(XYZ, ts))/180*pi* [0,0,1]);
    end
    % Mesh
    [fens,gcells] = H8_block(L,W,t,nL,nW,nt);
    [fens,gcells] = H8_to_H20(fens,gcells);
    % Material
    prop = property_linel_ortho (...
        struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23,'rho',1.5e-9)); %% 1627 kg/m^3
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss (struct ('mater',mater,'gcells',gcells, ...
        'integration_rule',gauss_rule (3,2),'Rm',@LayerRm));
    
OmegaShift=(2*pi*40) ^ 2;

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out
% No EBC's: free-floating
% Not No EBC anymore::!!
    nl0=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
        'inflate',0.001*t))];
    nl1=[fenode_select(fens, struct('box', [L,L,-Inf,Inf,-Inf,Inf],...
        'inflate',0.001*t))];
    nw0=[fenode_select(fens, struct('box', [0,L,0,0,-Inf,Inf],...
        'inflate',0.001*t))];    
    nw1=[fenode_select(fens, struct('box', [0,L,W,W,-Inf,Inf],...
        'inflate',0.001*t))];
    
    u   = set_ebc(u,nl0,nl0*0+1, [3], nl0*0);
    u   = set_ebc(u,nl1,nl1*0+1, [3], nl1*0);
    u   = set_ebc(u,nw0,nw0*0+1, [3], nw0*0);
    u   = set_ebc(u,nw1,nw1*0+1, [3], nw1*0);
             
    u   = apply_ebc (u);
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=stiffness(feb, geom, u);
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
% mat=get(ems,'mat');
% for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
%     Me=mat{j};
%     em2=sum(sum(Me));
%     dem2=sum(diag(Me));
%     mat{j}=diag(diag(Me)/dem2*em2);
% end
% ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
M = assemble (M, ems);

disp(['Number of DOF =' num2str(get(u, 'neqns'))]);

%
neigvs = 10;
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 2;
[W,Omega]=eigs(get(K,'mat')+OmegaShift*get(M,'mat'),get(M,'mat'),neigvs,'SM', options);
Omega =diag(diag(Omega)-OmegaShift);
[Ignore,ix] =sort (diag(Omega));



for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [   -1.0612
        -0.8377
        1.0948
        -0.0775
        0.0025
        0.0093
        0
        0
        0.0010
        0.0033];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[mean(box(1:2)), mean(box(3:4))-mean(box(1:2)), mean(box(5:6))];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=1365;
    w=clone(u,'w'); % make a copy of u
    
    for i=4:7
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
%         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,[]);
%             camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18));
            draw_axes (gv,struct('length',20 ));
            pause(0.005);
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
%         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',20 ));
        pause(2.55);
        
    end
end

end