disp('Taut wire, simple support--free: characteristic frequencies');
L=0.5;
P=20;
mu= 0.0150;
neigvs=5;
ns=[5,10,20,40]; % number of elements
% ns=[32]; % number of elements
eigenvalue_errors = zeros(length(ns),neigvs);
% Mesh
for mesh=1:length(ns)
    n=ns(mesh);
    [fens,gcells]= L2_block(L,n, 1.0);
    % Finite element block
    feb = feblock_defor_taut_wire(struct ('mater',mater,...
        'gcells',gcells,...
        'integration_rule',gauss_rule(1, 4),...%simpson_1_3_rule,...
        'P',P,'mu',mu));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 1, 'fens',fens));
    % Define the displacement field
    w   = 0*clone(geom,'w');
    % Apply EBC's
    fenids=[1]; prescribed=[1]; component=[1]; val=[0];
    w   = set_ebc(w, fenids, prescribed, component, val);
    w   = apply_ebc (w);
    % Number equations
    w   = numbereqns (w);
    % Assemble the stiffness matrix
    K = start (sparse_sysmat, get(w, 'neqns'));
    K = assemble (K, stiffness(feb, geom, w));
    % Assemble the consistent mass matrix
    M = start (sparse_sysmat, get(w, 'neqns'));
    ems = mass(feb, geom, w);
    % Assemble 
%     M= assemble (M, lumped_hrz(feb,ems)); %the lumped mass matrix
    M = assemble (M,ems);% or the consistent
    % keep only one of the above lines
    % Solve
    [W,Omega]=eigs(get(K,'mat'),get(M,'mat'),neigvs,'SM');
    [Omegas,ix]=sort(diag(Omega));

    scale=10;
    for i=1:neigvs
        analyt_eigenvalue =(P/mu*((2*i-1)/2*pi/L) ^ 2);
        eigenvalue_errors(mesh,i) =...
            (sqrt(Omegas(i))/2/pi-sqrt(analyt_eigenvalue)/2/pi)/(sqrt(analyt_eigenvalue)/2/pi);
        disp(['  Eigenvector ' num2str(i) ...
            ' eigenvalue ' num2str(sqrt(Omegas(i))/2/pi) ' versus '...
            num2str(sqrt(analyt_eigenvalue)/2/pi)]);
    end
     figure;
    for i=1:neigvs
       subplot(neigvs,1,i);
        w = scatter_sysvec(w, W(:,ix(i)));
        plot (gather (geom, (1:n+ 1),'values'), ...
            gather (w, (1:n+ 1),'values'),'ro-','linewidth', 2);
        hold on
        figure (gcf); set(gcf,'Visible','off');; set(gcf,'Visible','on');
        pause(2); % next eigenvector

    end
end
figure;
plot (ns,eigenvalue_errors,'linewidth', 3)
set(gca,'FontSize', 14) 
xlabel('Number of elements [ND]');        ylabel('(f_h-f_a)/f_a');
               
reference_eigenvalue_errors =[
     0.004117250605580   0.037294663725311   0.102657790843645   0.181063432023311   0.181522845130872
   0.001028398433798   0.009276610518994   0.025859084883559   0.050803157805495   0.083688639089555
   0.000257040727744   0.002314765825954   0.006437303093881   0.012636953395295   0.020927156090096
   0.000064256474992   0.000578397026594   0.001607145719690   0.003151409860933   0.005212479502109];

assignin('caller','faesor_test_passed',((norm(reference_eigenvalue_errors-eigenvalue_errors))<1e-9));
