disp('Taut wire: characteristic frequencies');
L=0.5;
P=20;
mu= 0.0150;
neigvs=5;
ns=[10,20,40]; % number of elements
% ns=[32]; % number of elements
eigenvalue_errors = zeros(length(ns),neigvs);
% Mesh
for mesh=1:length(ns)
    n=ns(mesh);
    [fens,gcells]= L2_block(L,n, 1.0);
    % Finite element block
    feb = feblock_defor_taut_wire(struct ('mater',mater,...
        'gcells',gcells,...
        'integration_rule',gauss_rule(1, 4),...%simpson_1_3_rule,...
        'P',P,'mu',mu));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 1, 'fens',fens));
    % Define the displacement field
    w   = 0*clone(geom,'w');
    % Apply EBC's
    fenids=[1,n+1]; prescribed=[1, 1]; component=[1, 1]; val=[0,0];
    w   = set_ebc(w, fenids, prescribed, component, val);
    w   = apply_ebc (w);
    % Number equations
    w   = numbereqns (w);
    % Assemble the stiffness matrix
    K = start (sparse_sysmat, get(w, 'neqns'));
    K = assemble (K, stiffness(feb, geom, w));
    % Assemble the mass
    M = start (sparse_sysmat, get(w, 'neqns'));
    ems = mass(feb, geom, w);
    M= assemble (M, lumped_hrz(feb,ems)); %the lumped mass matrix
    %     M = assemble (M,ems);% or the consistent
    % keep only one of the above lines
    % Solve
    [W,Omega]=eigs(get(K,'mat'),get(M,'mat'),neigvs,'SM');
    [Omegas,ix]=sort(diag(Omega));

    scale=10;
    for i=1:neigvs
        analyt_eigenvalue =(P/mu*(i*pi/L) ^ 2);
        eigenvalue_errors(mesh,i) =...
            (sqrt(Omegas(i))/2/pi-sqrt(analyt_eigenvalue)/2/pi)/(sqrt(analyt_eigenvalue)/2/pi);
        disp(['  Eigenvector ' num2str(i) ...
            ' eigenvalue ' num2str(sqrt(Omegas(i))/2/pi) ' versus '...
            num2str(sqrt(analyt_eigenvalue)/2/pi)]);
        figure;
        w = scatter_sysvec(w, W(:,ix(i)));
        plot (gather (geom, (1:n+ 1),'values'), ...
            gather (w, (1:n+ 1),'values'),'ro-','linewidth', 3);
        hold on
        figure (gcf)
        pause(0.2); % next eigenvector

    end
end
figure
plot (ns,eigenvalue_errors,'linewidth', 3)
set(gca,'FontSize', 14) 
xlabel('Number of elements [ND]');        ylabel('(f_h-f_a)/f_a');
                
             
reference_eigenvalue_errors =[
       -0.004107264756457  -0.016368356916552  -0.036602237995902  -0.064510716211379  -0.099683683842911
  -0.001027766751479  -0.004107264756458  -0.009227104011927  -0.016368356916552  -0.025504641595586
  -0.000257001130761  -0.001027766751481  -0.002311583809532  -0.004107264756458  -0.006413148855813];

assignin('caller','faesor_test_passed',((norm(reference_eigenvalue_errors-eigenvalue_errors))<1e-9));
