% The first bending mode with H20 28x28 mesh: 12.8177 radians per second
E=206000;% Pa
nu=0.3;
rho=7.85e-9;% tonne per millimeter cube
%     stabfact = 0.01;
L= 10e3; W= 10e3; H= 0.238e3;
Mshift =0;
neigvs = 8;

[fens,gcells] = block (L,W,H, 4, 4,3);
[fens,gcells] = H8_to_H20 (fens,gcells);

[fens, gcells]=renum_mesh(fens, gcells,'symamd');
disp (['Boundary: ' sprintf('%d/%d/%d %d:%02d:%02.3f',clock)])
bdry_gcells = mesh_bdry(gcells, []);
rl=gcell_select(fens,bdry_gcells,struct ('box',[-Inf Inf -Inf Inf H H],'inflate',0.0005));
bdry_gcells =subset(bdry_gcells,rl);

% Material
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's
enl=fenode_select (fens,struct ('box',[0 0 -Inf Inf -Inf Inf],'inflate',0.005));
u = set_ebc(u, enl, enl*0+1, [], enl*0);
u = apply_ebc (u);
% Number equations
u   = numbereqns (u);
%     clear fens gcells
feb = feblock_defor_ss (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule (3,2)));


% Assemble the system matrix
disp (['Mass: ' sprintf('%d/%d/%d %d:%02d:%02.3f',clock)])
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
Md=zeros(get(u, 'neqns'),1);
Me =get(ems,'mat');
eqnums =get(ems,'eqnums');
for i= 1:length(Me)
    %Hinton, Rock, Zienkiewicz lumping
    em2=sum(sum(Me{i}));
    dem2=sum(diag(Me{i}));
    Me{i} =diag(diag(Me{i})/dem2*em2);
    ene =eqnums{i};
    emask=ene>0;
    vene=ene(emask);
    if (~isempty(vene))
        Md(vene) =Md(vene) + diag(Me{i}(emask,emask));
    end
end
clear ems Me mat
M= sparse((1:get(u, 'neqns')),(1:get(u, 'neqns')),Md);
disp (['     Finished ' sprintf('%d/%d/%d %d:%02d:%02.3f',clock)])
disp (['Stiffness: ' sprintf('%d/%d/%d %d:%02d:%02.3f',clock)])
K = start (sparse_chol_altassem_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(feb, geom, u));
K = assemble_diag (K, Mshift*diag(M));
% Km=get(K,'mat');

options.disp =2;
options.issym =1;
options.p =2*neigvs;
[W,Omega]=eigs(@(b) (K\b),get(u, 'neqns'),M,neigvs,'SM',options);

[Omegas,ix]=sort(diag(Omega));
W=W(:,ix);
Omega= diag(Omegas);
disp('Dry Natural frequencies [radians per second]')
disp(num2str(sqrt(diag(Omega)')))