function sphshl_img_modes(R,t,n_R,Es,nus,rhos,neigvs,graphics,saveframes)
    try,avw_blank;catch,error('AVWTk is required; run AVWTk_init');end
    if (~exist('R'))
        R=500;% Internal radius of the Spherical shell, millimeters
    end
    if (~exist('t'))
        t=70;% Thickness of the sphere, millimeters
    end
    if (~exist('n_R'))
        n_R=8;% number of edges along the radius of the sphere
    end
    if (~exist('Es'))
        Es= 69000;%Aluminum material parameters, [MPa]
    end
    if (~exist('rhos'))
        rhos= 2.73e-9;% Aluminum material parameters, [1000*kg/mm^3]
    end
    if (~exist('nus'))
        nus=0.33;%Aluminum material parameters
    end
    if (~exist('neigvs'))
        neigvs = 14;% number of eigenvectors to compute
    end
    if (~exist('graphics'))
        graphics = 0;% Online graphics?
    end
    if (~exist('saveframes'))
        saveframes = 0;% save frames?
    end
    
    dim=  2*n_R*[1,1,1];% dimensions in pixels
    pixdim =2*R./dim;
    mkoptions.dim= dim;
    mkoptions.pixdim =pixdim;
    mkoptions.fill_pixel =0;
    mkoptions.name =mfilename;
    avw= avw_blank(mkoptions);
    mkoptions.R =R;
    mkoptions.center = pixdim.*dim/2;
    mkoptions.fill_pixel =1;
    avw=avw_fill_sphere(avw,mkoptions);
    mkoptions.R =R-t;
    mkoptions.center = pixdim.*dim/2;
    mkoptions.fill_pixel =0;
    avw=avw_fill_sphere(avw,mkoptions);
    
    [h,v,hmid] = voximg_to_H8(avw.img,1);
    v(:,1)=pixdim(1)*v(:,1);v(:,2)=pixdim(2)*v(:,2);v(:,3)=pixdim(3)*v(:,3);% Scale
    v= v-ones(size(v,1),1)*(max(v )+min(v ))/2;% Re-Center
    fens=fenodeset(struct('id',(1:size(v,1))','xyz',v));
    gcells =gcellset_H8(struct ('conn',h));
    
    
    %Shift to be able to solve for the frequencies of a free-floating
    %sphere
    kappa_T= 2.5;% lowest (torsional) mode of a solid sphere
    c_T=sqrt(Es/rhos /2/(1+ nus));
    f=kappa_T*c_T/(2*pi*R);
    OmegaShift=(f/100) ^ 2;
    
    % Eigenvalue solver options
    EVoptions.issym= true;
    EVoptions.isreal= true;
    EVoptions.tol=eps;
    EVoptions.maxit= 500;
    EVoptions.disp= 0;
    
    
    
    febh=@feblock_defor_ss_sri;
    gr=gauss_rule(3,[1,2]);
    
    % gv=drawmesh({fens,gcells},'gcells','facecolor','white')
    
    % Renumber the elements to minimize the fill in
    [fens, gcells]=renum_mesh(fens, gcells, 'symrcm');
    
    % Extract the boundary cells
    boundary_gcells=mesh_bdry(gcells);
    % Select only the cells on the outer surface of the Shell
    clear options
    options. facing=true;
    options. direction=@(x)(x);
    Extl=gcell_select(fens, boundary_gcells, options);
    boundary_gcells=subset(boundary_gcells, Extl );
    %gv=drawmesh({fens,boundary_gcells},'gcells','nodes','facecolor','white','facealpha',0.3,'shrink',0.8)
    
    % Material
    prop = property_linel_iso (struct('E',Es,'nu',nus,'rho',rhos));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = febh (struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',gr,'Rm', []));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
    % Compute volume
    %     V=measure(feblock(struct('gcells',gcells,'integration_rule',gauss_rule(3,2))),geom,@(x)1)
    %
    
    % Define the displacement field
    u   = 0*geom; % zero out
    % No EBC's: free-floating
    u   = apply_ebc (u);
    % Number equations
    u   = numbereqns (u);
    
    % Assemble the system matrix
    K = start (sparse_sysmat, get(u, 'neqns'));
    evs=stiffness(feb, geom, u);
    K = assemble(K, evs);
    clear evs
    M = start (sparse_sysmat, get(u, 'neqns'));
    ems=mass(feb, geom, u);
    %% If lumped mass matrix, uncomment the block below
    mat=get(ems,'mat');
    for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
        Me=mat{j};
        em2=sum(sum(Me));
        dem2=sum(diag(Me));
        mat{j}=diag(diag(Me)/dem2*em2);
    end
    ems=set(ems,'mat',mat);
    %% If lumped mass matrix, uncomment the block above
    % % Assemble mass matrix
    M = assemble (M, ems);
    disp(['Number of DOF =' num2str(get(u, 'neqns'))]);
    clear ems
    
    % Make sure the mass and stiffness matrices are symmetric
    mK=get(K,'mat'); clear K
    mK=(mK+mK')/2;
    mM=get(M,'mat'); clear M
    mM=(mM+mM')/2;
    %
    % Adjust the number of eigenvalues to be solved for the account for the rigid body modes
    [W,Omega]=eigs(mK+OmegaShift*mM,mM,neigvs+6,'SM', EVoptions);
    % Orthogonalize the eigenvectors
    [W,Ignore]=qr(W,0);
    % Orthonormalize the eigenvectors
    for j=1:size(W, 2)
        fac=W(:,j)'*mM*W(:,j);
        if (abs(fac)>0)
            W(:,j)=W(:,j)/sqrt(fac);
        else
            error(['Zero modal mass for mode ' num2str(j)])
        end
    end
    % Undo the shift
    if (min(real(diag(Omega)))<0)
        error('Shift might have led to missing low eigenvalues');
    end
    Omega =diag(real(diag(Omega))-OmegaShift);
    % Sort the eigenvalues and eigenvectors, and remove the rigid body modes
    [Ignore,ix] =sort (diag(Omega));
    Omegas=diag(Omega(ix(7:end),ix(7:end)));
    W=real(W(:,ix(7:end)));
    clear Omega ix Ignore
    
    for i=1:length(Omegas)
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]']);
    end
    
    
    % Plot
    if graphics
        animation=false;
        Camera =1.0e+003 * [[-1.0133   -0.8612    1.1200   -0.0296   -0.0210    0.0345]*(R/500)       0         0    0.0010    0.0333];
        xyz=get(geom,'values');
        box =reshape([min(xyz);max(xyz)],1,6);
        annloc=[mean(box(1:2))-R/2, min(box(3:4))+0, -R/2];
        bg=mesh_bdry(gcells);
        bfeb = feblock(struct ('gcells',bg));
        gv=graphic_viewer;
        gv=reset (gv,struct('limits',  limits));
        set_graphics_defaults
        w=clone(u,'w'); % make a copy of u
        % Take a snapshot of the mesh
        % First a static display
        gv=reset (gv,[]);
        camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom, 'u', 0*w,'facecolor','b','edgecolor',[1,1,1]*0.7, 'linewidth',2));
        axis off
        draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
        pause(1);
        %         saveas(gcf,[mfilename '-mesh.png']);
        %
        scale=R*10/2;
        % compute normals to the surface
        normals=get(geom,'values');
        for j=1:size(normals, 1)
            nn=norm(normals(j,:));
            if (nn>0)
                normals(j,:) = normals(j,:)'/nn;
            else
                normals(j,:) = [0,0,0];
            end
        end
        wmagmax=0;
        for i=1:length(Omegas)
            w = scatter_sysvec(w, W(:,i));
            w = w*(1/norm(w ));
            wv = get(w,'values');
            wmag =zeros(size(wv, 1),1 );
            for j=1:size(wv, 1)
                wmag(j) = wv(j,:)*normals(j,:)';
            end
            wmagmax=max( [wmagmax, max(wmag )] );
        end
        for i=1:length(Omegas)
            disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]']);
            w = scatter_sysvec(w, W(:,i));
            w = w*(1/norm(w ));
            wv = get(w,'values');
            wmag =zeros(size(wv, 1),1 );
            for j=1:size(wv, 1)
                wmag(j) = abs(wv(j,:)*normals(j,:)');
            end
            wmagf = field(struct ('name',['wmagf'], 'dim', 1, 'data',wmag));
            dcm=data_colormap(struct ('range',[0,+wmagmax], 'colormap',jet));
            colors=map_data(dcm, wmag);
            colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
            % First a static display
            gv=reset (gv,[]);
            camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', scale*w,'colorfield',colorfield,'edgecolor',[1,1,1]*0.7, 'linewidth',2));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]'],...
                struct('fontsize', 18, 'fontname', 'Times'));
            axis off
            draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
            draw_colorbar(gv,struct('colormap',get(dcm,'colormap'),...
                'position',[0.87, 0.24, 0.05, 0.5],...
                'minmax', get(dcm,'range')/wmagmax,...
                'label','$|u_n|$', 'fontname', 'Times', 'interpreter', 'latex'));
            pause(1);
            if (saveframes)
                saveas(gcf,[mfilename '-n_R'  num2str(n_R) '-m' num2str(i) '.eps'],'epsc');
            end
            if (animation)
                % Now a dynamic one
                for xscale=scale*sin((0:1:42)/21*2*pi)
                    gv=reset (gv,[]);
                    camset(gv,Camera);
                    draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield,'edgecolor',[1,1,1]*0.7, 'linewidth',2));
                    %                 for  isovalue=linspace(min(wmag),max(wmag),20)
                    %                     draw_isosurface(bfeb,gv, struct ('x', geom,'u',xscale*w,'scalarfield',wmagf,'isovalue',isovalue,'color',[0,0,0]));
                    %                 end
                    draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omegas(i))/2/pi) ' [Hz]'],...
                        struct('fontsize', 18, 'fontname', 'Times'));
                    draw_axes (gv,struct('length',R/3,'origin', [-R,+R,-R]));
                    axis off
                    pause(0.005);
                end
                pause(0.155);
            end
        end
    end
end
