function bending_wave_expl
% clear all classes
graphics = false;
igraphics=(1:250:40000000);%+5000;
plots = true;
% Parameters:
u=physical_units_struct;
E=205000*u.MEGA*u.PA;
nu=0.3;
rho  = 7850*u.KG/u.M^3;
% eta=2000*u.PA*u.SEC;
loss_tangent=0.0001;
G=E/2/(1+nu);
frequency = 1/0.0058;
eta = loss_tangent*G/(2*pi*frequency);
L = 200*u.MM;
W = 4*u.MM;
H = 8*u.MM;
tolerance  =W/1000;
vmag = 0.1*u.M/u.SEC;
tfinal = 0.03*u.SEC;
scale=4000;
stressscale=scale/20;

[fens,gcells] = H8_block(L,W,H, 50,1,4);


% Material
prop = property_linviscel_iso (struct('E',E,'nu',nu,'rho',rho,'eta',eta));
mater = mater_defor_ss_linviscel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss(struct ('mater',mater, 'gcells',gcells, 'integration_rule',gauss_rule (3,2)));

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's
%Clamped end
ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',tolerance));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);
 
% Initial conditions
v = u;% copy everything from displacement field to velocity
vv =get(v,'values' );
fenids=fenode_select (fens,struct('box',[0,L-10*tolerance,-Inf,Inf,-Inf,Inf],'inflate',tolerance));
vv (fenids,3) =vmag;
v =set(v,'values',vv);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(feb, geom, u));
C = start (sparse_sysmat, get(v, 'neqns'));
C = assemble (C, damping(feb, geom, v));
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
mat=get(ems,'mat');
for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    Me=mat{j};
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    mat{j}=diag(diag(Me)/dem2*em2);
end
ems=set(ems,'mat',mat);
% Consistent or lumped mass matrix
M = assemble (M, ems);

% Solve
Kmat = get(K,'mat');
Cmat = get(C,'mat');
Mmat = get(M,'mat');
U0 = gather_sysvec(u);
V0= gather_sysvec(v);
t=0;
o2=eigs(Kmat,Mmat,1,'LM');
dt= 0.99* 2/sqrt(o2)
if graphics
    gv=graphic_viewer;
    gv=reset (gv,[]);
elseif plots
    pf = figure(gcf);
end
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, nz_ebc_loads(feb, geom, u));
% F = assemble (F, nz_ebc_loads_damping(feb, geom, v));
A0 =Mmat\(get(F,'vec'));
snapshot =0;
% h = waitbar(0.0,'Running...','CreateCancelBtn','if ~getappdata(gcf,''stop'') setappdata(gcf,''stop'',true); else delete(gcf); end');
% set(h,'Name',mfilename);
% setappdata(h,'stop',false);
corner=fenode_select (fens,struct('box',[0 0 0 0 0 0],'inflate',tolerance));
corneru= [];
step =0;
while t <tfinal
    step = step  +1;
    t=t+dt;
    %     waitbar((t/tfinal),h,['Time = ' num2str(t)]);
    %     if getappdata(h,'stop') break; end
    U1 = U0 +dt*V0+(dt^2)/2*A0;
    F = start (sysvec, get(u, 'neqns'));
%     F = assemble (F, nz_ebc_loads(feb, geom, u));
    R=get(F,'vec')-Kmat*U1-Cmat*(V0+dt/2*A0);
    A1 = Mmat\(R-dt/2*Cmat*A0);
    it=1;
    while true
        pA1=A1;
        A1 = Mmat\(R-(dt/2)*Cmat*A1);
        %             disp([' Iteration ' num2str(it) ': ' num2str(norm(pA1-A1))])
        if norm(pA1-A1)<=0.0001*norm(A1)
            break;
        end
        it=it+1;
    end
    V1 = V0 +dt/2* (A0+A1);
    U0 = U1;
    V0 = V1;
    A0 = A1;
    u = scatter_sysvec(u, U1);
    v = scatter_sysvec(v, V1);
    corneru= [corneru    gather(u, corner,'values')];
    if graphics && (~isempty(igraphics) && (igraphics(1)==step))
        %         clear(gv);
        gv=reset (gv,[]);
        draw(feb,gv, struct ('x', geom, 'u', scale*u,'facecolor','none','edgecolor','blue'));
        view(3); pause(0.1);
        %             saveas(gcf, [mfilename '-' num2str(etaw) '-' num2str(snapshot) '.png'], 'png');
        snapshot = snapshot +1;
        igraphics= igraphics(2:end);
     elseif plots && (~isempty(igraphics) && (igraphics(1)==step))
         figure (pf);
         %             plot (corneru(1,:),corneru(2,:),'g'); hold on
         plot(t,corneru(3,end),'bx','Markersize',4); hold on
         %  plot(t,(V1'*Mmat*V1+U1'*Kmat*U1)/2,'ro','Markersize',4); hold on
         igraphics= igraphics(2:end);
     end
end
end