function bending_wave_impl
% clear all classes
graphics = false;
igraphics=(1:2:40000000);%+5000;
plots = true;
% Parameters:
u=physical_units_struct;
E=205000*u.MEGA*u.PA;
nu=0.3;
rho  = 7850*u.KG/u.M^3;
eta=2000*u.PA*u.SEC;
loss_tangent=0.01;
G=E/2/(1+nu);
frequency = 1/0.0058;
eta = loss_tangent*G/(2*pi*frequency);
L = 200*u.MM;
W = 4*u.MM;
H = 8*u.MM;
tolerance  =W/1000;
vmag = 0.1*u.M/u.SEC;
tfinal = 3*u.SEC;
scale=4000;
stressscale=scale/20;

[fens,gcells] = H8_block(L,W,H, 50,1,4);

% [fens,gcells] = H8_to_H20(fens,gcells);

% Material
prop = property_linviscel_iso (struct('E',E,'nu',nu,'rho',rho,'eta',eta));
mater = mater_defor_ss_linviscel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss(struct ('mater',mater, 'gcells',gcells, 'integration_rule',gauss_rule (3,2)));

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's
%Clamped end
ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',tolerance));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);
 
% Initial conditions
v = u;% copy everything from displacement field to velocity
vv =get(v,'values' );
fenids=fenode_select (fens,struct('box',[0,L-10*tolerance,-Inf,Inf,-Inf,Inf],'inflate',tolerance));
vv (fenids,3) =vmag;
v =set(v,'values',vv);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(feb, geom, u));
C = start (sparse_sysmat, get(v, 'neqns'));
C = assemble (C, damping(feb, geom, v));
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
mat=get(ems,'mat');
for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    Me=mat{j};
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    mat{j}=diag(diag(Me)/dem2*em2);
end
ems=set(ems,'mat',mat);
% Consistent or lumped mass matrix
M = assemble (M, ems);

% Solve
Kmat = get(K,'mat');
Cmat = get(C,'mat');
Mmat = get(M,'mat');
U0 = gather_sysvec(u);
V0= gather_sysvec(v);
t=0;
o2=eigs(Kmat,Mmat,1,'LM');
dt= 560*0.99* 2/sqrt(o2)
if graphics
    gv=graphic_viewer;
    gv=reset (gv,[]);
elseif plots
    pf = figure(gcf);
end
F0 = start (sysvec, get(u, 'neqns'));
F0 = assemble (F0, nz_ebc_loads(feb, geom, u));
L0 = get(F0,'vec');
snapshot =0;
h = waitbar(0.0,'Running...','CreateCancelBtn','if ~getappdata(gcf,''stop'') setappdata(gcf,''stop'',true); else delete(gcf); end');
set(h,'Name',mfilename);
setappdata(h,'stop',false);
corner=fenode_select (fens,struct('box',[0 0 0 0 0 0],'inflate',tolerance));
corneru= [];
step =0;
while t <tfinal
    step = step  +1;
    t=t+dt;
    waitbar((t/tfinal),h,['Time = ' num2str(t)]);
    if getappdata(h,'stop') break; end
    F1 = start (sysvec, get(u, 'neqns'));
    F1 = assemble (F1, nz_ebc_loads(feb, geom, u));
    L1 = get(F1,'vec');
    V1 = (Mmat + (dt/2)*Cmat + ((dt/2)^2)*Kmat)\(Mmat*V0 - Cmat*(dt/2*V0) - Kmat*((dt/2)^2)*V0 - dt*Kmat*U0 + (dt/2)*(L0+L1));%
    U1 = U0 +dt/2*(V0+V1);
    U0 = U1;
    V0 = V1;
    L0 = L1;
    u = scatter_sysvec(u, U1);
    v = scatter_sysvec(v, V1);
    corneru= [corneru    gather(u, corner,'values')];
    if graphics && (~isempty(igraphics) && (igraphics(1)==step))
        %         clear(gv);
        gv=reset (gv,[]);
        draw(feb,gv, struct ('x', geom, 'u', scale*u,'facecolor','none','edgecolor','blue'));
        view(3); pause(0.1);
        %             saveas(gcf, [mfilename '-' num2str(etaw) '-' num2str(snapshot) '.png'], 'png');
        snapshot = snapshot +1;
        igraphics= igraphics(2:end);
     elseif plots && (~isempty(igraphics) && (igraphics(1)==step))
        figure (pf);
        %             plot (corneru(1,:),corneru(2,:),'g'); hold on
                plot(t,corneru(3,end),'bx','Markersize',4); hold on
%         plot(t,(V1'*Mmat*V1+U1'*Kmat*U1)/2,'bx','Markersize',4); hold on
        igraphics= igraphics(2:end);
    end
end
end