function onedtraveling
disp('Pressure wave');
% clear all classes
graphics = ~false;
igraphics=(1: 20:4000);%+5000;
plots = false;
% Parameters:
bulk = 2100;
etaw=20e-6;
frequency = 3500;
circular_frequency=2*pi*frequency;
rho = 1e-9;
c=sqrt (bulk/rho); % speed of sound in water: mm*s^-1
wavenumber = circular_frequency/c;
pressuremag = 10^9*1e-6*1e-6; % N*mm^-2
velmag = pressuremag/rho/c; % mm*s^-1
dt =  8.5604e-006;
tfinal = 5e-2;
uampl= velmag/circular_frequency;
vampl=velmag;
scale = 500000;
nvalsrange=[-0.00064, 0.00064];
Wb=200;
Hb= 26;
limits =[-0.1*Wb 3.1*Wb  -0.1*Hb 1.1*Hb scale*nvalsrange(1) scale*nvalsrange(2)];
tic
n=3;
% Mesh'
[fens,gcells]=Q4_block(3*Wb, Hb, 3*n*8, 1, struct('other_dimension',1.0));
% Material
propw = property_viscous_fluid(struct('bulk',bulk,'eta', etaw,'rho',rho));
matw = mater_defor_ss_linviscel_biax (struct('property',propw, ...
    'reduction','strain'));
% Finite element blocks
febw = feblock_defor_ss(struct ('mater',matw, 'gcells',gcells,...
    'integration_rule',gauss_rule (2, 2)));
options =struct ('inflate', 1/100,'anynode', true);
options.box = [0 0  0  Hb];
bcl=gcell_select(fens,gcells,options);
options.box = [3*Wb 3*Wb  0  Hb];
bcl=[bcl gcell_select(fens,gcells,options)];
bcl=unique(bcl);
febwb = feblock_defor_ss(struct ('mater',matw, 'gcells',subset(gcells,bcl),...
    'integration_rule',gauss_rule (2, 2)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 2, 'fens',fens));
% Define the displacement field
u   = clone(geom,'u');
u   = u*0; % zero out
% Apply EBC's
ebc_fenids=fenode_select (fens,struct('box',[0 3*Wb 0 0],'inflate',1/100));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_prescribed*0+2;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[0 3*Wb Hb Hb],'inflate',1/100));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=ebc_prescribed*0+2;
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);
v = u;% copy everything from displacement field to velocity
ebc_fenids0=fenode_select (fens,struct('box',[0 0 0 Hb],'inflate',1/100));
ebc_fenidsL=fenode_select (fens,struct('box',[3*Wb 3*Wb 0 Hb],'inflate',1/100));
    function recalculate_ebc(t)
        ebc_prescribed=ones(1,length (ebc_fenidsL));
        ebc_a=ones(1,length (ebc_fenidsL));
        ebc_comp=ebc_prescribed*0+1;
        ebc_val=uampl*sin(wavenumber*(c*t))*ebc_a;
        u   = set_ebc(u, ebc_fenidsL, ebc_prescribed, ebc_comp, ebc_val);
        u   = apply_ebc (u);
        ebc_val=wavenumber*c*uampl*cos(wavenumber*(c*t))*ebc_a ;
        v   = set_ebc(v, ebc_fenidsL, ebc_prescribed, ebc_comp, ebc_val);
        v   = apply_ebc (v);
        active = (t>3*Wb/c);
        ebc_prescribed=ones(1,length (ebc_fenids0));
        ebc_a=ones(1,length (ebc_fenids0));
        ebc_comp=ebc_prescribed*0+1;
        ebc_val=active*uampl*sin(wavenumber*(-3*Wb +c*t))*ebc_a;
        u   = set_ebc(u, ebc_fenids0, ebc_prescribed, ebc_comp, ebc_val);
        u   = apply_ebc (u);
        ebc_val=active*wavenumber*c*uampl*cos(wavenumber*(-3*Wb+c*t))*ebc_a;
        v   = set_ebc(v, ebc_fenids0, ebc_prescribed, ebc_comp, ebc_val);
        v   = apply_ebc (v);
    end
recalculate_ebc(0);
corner=fenode_select (fens,struct('box',[0 0 Hb Hb],'inflate',1/100));
corneru= [];
% Number equations
u   = numbereqns (u);
% Initial conditions
v = u;% copy everything from displacement field to velocity
% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(febw, geom, u));
C = start (sparse_sysmat, get(v, 'neqns'));
C = assemble (C, damping(febw, geom, v));
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(febw, geom, u);
% If lumped mass matrix, uncomment this block
mat=get(ems,'mat');
for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
    Me=mat{j};
    em2=sum(sum(Me));
    dem2=sum(diag(Me));
    mat{j}=diag(diag(Me)/dem2*em2);
end
ems=set(ems,'mat',mat);
% Consistent or lumped mass matrix
M = assemble (M, ems);

% Solve
Kmat = get(K,'mat');
Cmat = get(C,'mat');
Mmat = get(M,'mat');
U0 = gather_sysvec(u);
V0= gather_sysvec(v);
t=0;
o2=eigs(Kmat,Mmat,1,'LM');
dt= 0.699* 2/sqrt(o2)
if graphics
    gv=graphic_viewer;
    gv=reset (gv,struct ('limits',limits));
elseif plots
    pf = figure(gcf);
end
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, nz_ebc_loads(febwb, geom, u));
F = assemble (F, nz_ebc_loads_damping(febwb, geom, v));
A0 =Mmat\(get(F,'vec'));
snapshot =0;
h = waitbar(0.0,'Running...','CreateCancelBtn','if ~getappdata(gcf,''stop'') setappdata(gcf,''stop'',true); else delete(gcf); end');
set(h,'Name',mfilename);
setappdata(h,'stop',false);
step =0;
while t <tfinal
    step = step  +1;
    t=t+dt;
    waitbar((t/tfinal),h,['Time = ' num2str(t)]);
    if getappdata(h,'stop') break; end
    U1 = U0 +dt*V0+(dt^2)/2*A0;
    recalculate_ebc(t);
    F = start (sysvec, get(u, 'neqns'));
    F = assemble (F, nz_ebc_loads(febwb, geom, u));
    F = assemble (F, nz_ebc_loads_damping(febwb, geom, v));
    R=get(F,'vec')-Kmat*U1-Cmat*(V0+dt/2*A0);
    A1 = A0;
    it=1;
    while true
        pA1=A1;
        A1 = Mmat\(R-dt/2*Cmat*A1);
        %             disp([' Iteration ' num2str(it) ': ' num2str(norm(pA1-A1))])
        if norm(pA1-A1)<=0.0001*norm(A1)
            break;
        end
        it=it+1;
    end
    V1 = V0 +dt/2* (A0+A1);
    U0 = U1;
    V0 = V1;
    A0 = A1;
    u = scatter_sysvec(u, U1);
    v = scatter_sysvec(v, V1);
    corneru= [corneru    gather(u, corner,'values')];
    if graphics & (~isempty(igraphics) & (igraphics(1)==step))
        %         clear(gv);
        gv=reset (gv,struct ('limits',limits));
        fld = field_from_integration_points(febw, geom, u, [], 'pressure',1);
        nvals=get(fld,'values');%min(nvals),max(nvals)
        %nvalsrange=[-0.000015, 0.000015];%[min(nvals),max(nvals)]
        dcm=data_colormap(struct ('range',nvalsrange, 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        uv =get(u,'values');
        geom3 = field(struct ('name',['geom3'], 'data', [get(geom,'values')+scale*uv,scale/2*nvals]));
        %             geom3 = field(struct ('name',['geom3'], 'data', [get(geom,'values')+scale*uv,10*scale*uv(:,1)]));
        draw(febw,gv, struct ('x', geom3, 'u', 0*geom3,...
            'colorfield',colorfield,'edgecolor','blue'));
        %                         draw(febw,gv, struct ('x', geom, 'u', scale*u,...
        %                             'facecolor','none','edgecolor','black'));
        %                         draw(febt,gv, struct ('x', geom, 'u', scale*u,...
        %                             'facecolor','none','edgecolor','red'));
        view(3); pause(0.1);
        %             saveas(gcf, [mfilename '-' num2str(etaw) '-' num2str(snapshot) '.png'], 'png');
        snapshot = snapshot +1;
        igraphics= igraphics(2:end);
    elseif plots
        figure (pf);
        %             plot (corneru(1,:),corneru(2,:),'g'); hold on
        plot(t,U1'*Kmat*U1/2+V1'*Mmat*V1/2,'ro'); hold on
    end
end
end