% Mesh of an entire sphere.
%
% function [fens,gcells] = H8_structured_sphere(R, Mesh_size)
%
% R= radius of the sphere, 
% Mesh_size= mesh size, 
%
% Example:
%    R= 0.5; Mesh_size=R/2;
%    [fens,gcells] = H8_structured_sphere(R, Mesh_size)
%    drawmesh ({fens,gcells}, 'gcells', 'facecolor','y');
%
function [fens,gcells] = H8_structured_sphere(R, Mesh_size)
    if (~exist('R','var'))
        R= 0.5;
    end
    if (~exist('Mesh_size','var'))
        Mesh_size=R/2;
    end
    
    nperradius=round(R/ Mesh_size);
    if (nperradius<1)
    	nperradius=1;
    end
    tol=R/nperradius/1000;% geometrical tolerance
    
    % Mesh
    [fens,gcells]=H8_sphere_n(R,nperradius);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,-1,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,0,-1], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
end