function [fens, gcells] = L2_block(Length, nL, options)
% Mesh of a line segment located in a given range, L2 Elements
%
% Syntax
%
% [fens,gcells] = L2_block(Length,nL,options)
%
% Description
%
% Creates a line divided into nL elements of the L2 type in the interval
% from zero to Length.
%
% Example
% 
% [fens,gcells] = L2_block(10, 5, struct('other_dimension', 0.1)); 
% drawmesh({fens,gcells},'nodes','gcells','facecolor','red')
%
% See also: L2_blockx, gcellset_L2

    % create an appropriate number of nodes along the length of the line
    % between each of which we will create L2 gcells
    xyz = linspace(0, Length, nL+1);
    
    % use more generic L2_blockx function to create the nodes at the
    % desired locations
    [fens, gcells] = L2_blockx(xyz', options);
    
end
