function  [fens, gcells] = L2_blockx(xyz, options)
% Mesh of a 1-D block, L2 cells. The nodes are located at the supplied
% coordinates.  This allows for construction of graded meshes.
%
% Syntax
%
% [fens,gcells] = L2_blockx(xyz,options)
%
% Input
%
% xyz - (n x p) matrix containing the locations of the individual nodes.
%
% options - structure with fields recognized by the constructor of the
%   gcell_L2 finite element
%
% Example
% 
% [fens,gcells] = L2_blockx((2:1:7)'.^3, struct('other_dimension', 0.1)); 
% drawmesh({fens,gcells},'nodes','gcells','facecolor','red')
%
% See also: L2_block, gcellset_L2

    if ~isstruct(options)
        
        other_dimension = options; 
         
        clear options;
        
        options.other_dimension = other_dimension;
        
    end
    
    ncells = length(xyz) - 1;
    nnodes = (ncells+1);

    % create the nodes
    fens = fenodeset(struct('id', (1:nnodes)', 'xyz', xyz));
    
    % create the cells
    options.conn = [(1:ncells)', (2:ncells+1)'];
    gcells = gcellset_L2(options);
    
end
