function [fens,gcells] = Q4_block(Length,Width,nL,nW,options)
% Mesh of a rectangle subdivided into smaller quadrilateral elements
%
% Syntax
%
% [fens,gcells] = Q4_block(Length,Width,nL,nW,options)
%
% Description
%
% Creates a mesh of Q4 rectangular elements located in a given range, The
% mesh is divided into nL elements in the first dimension and nW elements
% in the second dimension.
% 
% options is a structure with fields recognized by the gcellset_Q4
% constructor.
% 
% See the visual gallery by running test_block
% 
% Example
%
% [fens,gcells] = Q4_block(1, 1, 5, 3, []); 
% drawmesh({fens,gcells})
%
%
% See also: Q4_blockx, gcellset_Q4

    % create an appropriate number of nodes along the length of the line
    % between each of which we will create L2 gcells
    xs = linspace(0, Length, nL+1)';
    
    ys = linspace(0, Width, nW+1)';
    
    [fens,gcells] = Q4_blockx(xs,ys,options);

end
