% Create a volume mesh of 1/8 of the spherical shell of "radius" and "thickness". 
%
% function [fens,gcells]=Spherical_shell_mesh(radius,nrefine,thickness,nlayers, Convert)
%
% Create a mesh of 1/8 of the spherical shell of "radius"  and "thickness". The radius is the
% internal radius.  The external radius is radius + thickness.
% The  mesh will consist of
% three*nlayers hexahedral elements if "nrefine==0", or more if "nrefine>0".
% "nrefine" is the number of bisections applied  to refine the mesh.
% The shell has the thickness "thickness".
%
% The mesh is first created to consist of H8 Hexahedra.  It may be then converted to other 
% hexahedral types if Convert is a handle to a conversion function (for instance H8_to_H20).
%
function [fens,gcells]=Spherical_shell_mesh(radius,nrefine,thickness,nlayers, Convert)
p=1/3;
fens =fenodeset(struct('id',1:7,'xyz',[0, 0; 1,0; 0,1; 1/2,0; 1/2,1/2; 0,1/2; 1/2*(1-p),p]));
    gcells=gcellset_Q4(struct ('id', 1:3,'conn',[[1, 4, 7, 6];[4, 2, 5, 7]; [3, 6, 7, 5]],'other_dimension', thickness));
    for i=1:nrefine
        [fens,gcells]=Q4_refine(fens,gcells);
    end
    function xyz= z(xyz, layer)
        xyz= [xyz,layer/nlayers*thickness];
    end
	X=[+1,0,0; 0,+1,0; 0,0,+1; sqrt(2)/2,sqrt(2)/2,0; 0,sqrt(2)/2,sqrt(2)/2; sqrt(2)/2,0,sqrt(2)/2];
    [fens,gcells] = Q4_extrude(fens,gcells,nlayers,@z);
	if ~isempty(Convert)
	    [fens,gcells] = Convert (fens,gcells);
	end
    xs= get(fens,'xyz');
	Newxs=xs;
    for i=1:size(xs,1)
    	p= X'*T6bfun(xs(i,1:2));
		R= radius+xs(i,3);
    	Newxs(i,:)= R*p/ norm(p);
    end
    fens= set(fens,'xyz', Newxs);
end

function N = T6bfun(param_coords)
    r=param_coords(1);
    s=param_coords(2);
    t = 1 - r - s;
    N = [...
        t * (t + t - 1);
        r * (r + r - 1);
        s * (s + s - 1);
        4 * r * t;
        4 * r * s;
        4 * s * t];
end