% Tetrahedral (t4) Mesh of a block located in a given range
%
% function [fens,gcells] = t4block(Length,Width,Height,nL,nW,nH)
%
% Range =<0,Length> x <0,Width> x <0,Height>
% Divided into elements: nL, nW, nH in the first, second, and
% third direction (x,y,z).
function [fens,gcells] = T4_blob(Length,Width,Height,nL,nW,nH)
    nnodes=(nL+1)*(nW+1)*(nH+1);
    xyz= zeros(nnodes,3);
    f=1;
    for k=1:(nH+1)
        for j=1:(nW+1)
            for i=1:(nL+1)
                xyz(f,:)=[Length,Width,Height].*rand(1,3);
                f=f+1;
            end
        end
    end
    T = delaunayn(xyz);
    ncells=size(T,1);
    gcells=gcellset_T4(struct ('id', (1:ncells)','conn',T));
    fens=fenodeset (struct('id',(1:nnodes)','xyz',xyz));
end

%compute volume of a tetrahedron
% Given the 4x3 vertex coordinate matrix V of a tetrahedron, TETVOL(V)
% returns the volume of the tetrahedron.
function vol = tetvol(v)
    vol = det([v(2,:)-v(1,:);v(3,:)-v(1,:);v(4,:)-v(1,:)])/6;
    return;
end
