% Export mesh to a VTK 1.0 file as an unstructured grid.
%
% function vtk_export_mesh (theFile,Connectivity,Points,Cell_types,options)
%
% theFile= File name as a string,
% Connectivity= connectivity array, one row per cell,
% Points= Coordinate array, one row per point,
% Cell_types= Cell type code: L2=3, T3=5, Q4=9, T4=10, H8=12
function vtk_export_mesh (theFile,Connectivity,Points,Cell_types,options)
binary= ~true;
scalars=[]; scalars_name= ['Data'];
if ( exist ( 'options', 'var') )
    if (isfield(options, 'binary'))
        binary= options.binary;
    end
    if (isfield(options, 'scalars'))
        scalars= options.scalars;
    end
    if (isfield(options, 'scalars_name'))
        scalars_name= options.scalars_name;
    end
end
if (~iscell(Connectivity))
    Connectivity={Connectivity};
end
if (~iscell(Cell_types))
    Cell_types={Cell_types};
end
[pathstr, name, ext] = fileparts(theFile);
if (~strcmp(ext,'vtu'))
    theFile = [theFile '.vtu'];
end
fid=fopen(theFile,'w');
if (fid==-1)
    warning (['Could not open ' theFile])
    return
end
ncells =0;
for k=1:length(Connectivity)
    f=Connectivity{k};
    ncells =ncells +size(f, 1);
end
fprintf (fid, '<VTKFile type="UnstructuredGrid" version="0.1" byte_order="BigEndian">\n' );
fprintf (fid, '  <UnstructuredGrid>\n' );
fprintf (fid, '    <Piece NumberOfPoints="%d" NumberOfCells="%d">\n', size(Points, 1), ncells );

fprintf (fid, '      <Points>\n' );
if (~binary)
    fprintf (fid, '        <DataArray type="Float64" NumberOfComponents="3" Format="ascii">\n' );
    for i = 1 : size(Points, 1)
        fprintf (fid, '%g %g %g\n', Points(i,:));
    end
    fprintf (fid, '        </DataArray>\n' );
else
    fprintf (fid, '        <DataArray type="Float64" NumberOfComponents="3" Format="binary">\n' );
    encoder = org.apache.commons.codec.binary.Base64;
    base64string = char(encoder.encode(Points))';
    fwrite(fid,base64string);
    fprintf (fid, '        </DataArray>\n' );
end
fprintf (fid, '      </Points>\n' );

fprintf (fid, '      <Cells>\n' );
for k=1:length(Connectivity)
    f=Connectivity{k};
    ctype=zeros(size(f, 1),1)+Cell_types{k};
    if (~binary)
        fprintf (fid, '        <DataArray type="Int32" Name="connectivity" Format="ascii">\n' );
        for i= 1:size(f, 1)
            for j= 1: size(f,2)
                fprintf(fid,'%d ',f(i,j)-1);
            end
            fprintf(fid,'\n');
        end
    else
        fprintf (fid, '        <DataArray type="Int32" Name="connectivity" Format="binary">\n' );
        fwrite(fid,cast([f-1],'int32'),'int32');
    end
    fprintf (fid, '        </DataArray>\n' );
    
    if (~binary)
        fprintf (fid, '        <DataArray type="Int32" Name="offsets" Format="ascii">\n' );
        offset = 0;
        for i= 1:size(f, 1)
            offset = offset + size(f,2);
            fprintf (fid, '%d\n', offset);
        end
    else
        fprintf (fid, '        <DataArray type="Int32" Name="offsets" Format="binary">\n' );
        encoder = org.apache.commons.codec.binary.Base64;
        base64string = char(encoder.encode(cast((size(f,2):size(f,2):size(f,1)*size(f,2)+1),'uint8')))';
        fwrite(fid,base64string);
    end
    fprintf (fid, '        </DataArray>\n' );
    if (~binary)
        fprintf (fid, '        <DataArray type="Int32" Name="types" Format="ascii">\n' );
        for i= 1:size(f, 1)
            fprintf(fid,'%d\n',ctype(i));
        end
    else
        fprintf (fid, '        <DataArray type="Int32" Name="types" Format="binary">\n' );
        fwrite(fid,cast(ctype,'int32'),'int32');
    end
    fprintf (fid, '        </DataArray>\n' );
    
end
fprintf (fid, '      </Cells>\n' );

if (~isempty(scalars))
    fprintf(fid,'point_data %d\n',length(scalars));%
    fprintf(fid,'SCALARS %s float\n',scalars_name);%
    fprintf(fid,'LOOKUP_TABLE default\n');%
    if (~binary)
        for j= 1:length(scalars)
            fprintf(fid,'%s\n',scalars(j));
        end
    else
        fwrite(fid,cast(scalars,'single'),'single');
    end
end
fprintf(fid,'\n');
fprintf (fid, '    </Piece>\n' );
fprintf (fid, '  </UnstructuredGrid>\n' );
fprintf (fid, '</VTKFile>\n' );
fid=fclose(fid);
end
